@@ -304,6 +304,10 @@ static void show_procs(unsigned maxcmd, int do_header, int pflags, void* args, i
 
     /* initiate process table scan */
     tab = openproc(pflags, args, N);
+    if (!tab) {
+        fprintf(stderr, "Error: can not access /proc.\n");
+        exit(1);
+    }
 
     if (do_header) puts(mode[CL_fmt].header);	/* print header */
 
