@@ -1,9 +1,9 @@
 /***********************************************************************\
-*   Copyright (C) 1992 by Michael K. Johnson, johnsonm@sunsite.unc.edu	*
+*   Copyright (C) 1992-1998 by Michael K. Johnson, johnsonm@redhat.com  *
 *									*
-*	This file is placed under the conditions of the GNU public	*
-*	license, version 2, or any later version.  See file COPYING	*
-*	for information on distribution conditions.			*
+*	This file is placed under the conditions of the GNU Library	*
+*	General Public License, version 2, or any later version.	*
+*	See file COPYING for information on distribution conditions.	*
 \***********************************************************************/
 #include <stdlib.h>
 #include <stdio.h>
