@@ -14,6 +14,7 @@
 #include <stdarg.h>
 #include <string.h>
 #include <unistd.h>
+#include <signal.h>
 #include <fcntl.h>
 #include <sys/dir.h>
 #include <sys/types.h>
@@ -51,8 +52,10 @@ PROCTAB* openproc(int flags, ...) {
 /* terminate a process table scan
  */
 void closeproc(PROCTAB* PT) {
-    if (PT->procfs) closedir(PT->procfs);
-    if (PT)         free(PT);
+    if (PT){
+        if (PT->procfs) closedir(PT->procfs);
+        free(PT);
+    }
 }
 
 /* deallocate the space allocated by readproc if the passed rbuf was NULL
@@ -71,7 +74,7 @@ void freeproc(proc_t* p) {
 
 
 
-void status2proc (char* S, proc_t* P, int fill) {
+static void status2proc (char* S, proc_t* P, int fill) {
     char* tmp;
     if (fill == 1) {
         memset(P->cmd, 0, sizeof P->cmd);
@@ -144,14 +147,17 @@ void status2proc (char* S, proc_t* P, int fill) {
 
 
 /* stat2proc() makes sure it can handle arbitrary executable file basenames
-   for `cmd', i.e. those with embedded whitespace or embedded ')'s.  Such names
-   confuse %s (see scanf(3)), so the string is split and %39c is used instead.
-   (except for embedded ')' "(%[^)]c)" would work.
-*/
-void stat2proc(char* S, proc_t* P) {
+ * for `cmd', i.e. those with embedded whitespace or embedded ')'s.
+ * Such names confuse %s (see scanf(3)), so the string is split and %39c
+ * is used instead. (except for embedded ')' "(%[^)]c)" would work.
+ */
+static void stat2proc(char* S, proc_t* P) {
     int num;
     char* tmp = strrchr(S, ')');	/* split into "PID (cmd" and "<rest>" */
     *tmp = '\0';			/* replace trailing ')' with NUL */
+    /* fill in default values for older kernels */
+    P->exit_signal = SIGCHLD;
+    P->processor = 0;
     /* parse these two strings separately, skipping the leading "(". */
     memset(P->cmd, 0, sizeof P->cmd);	/* clear even though *P xcalloc'd ?! */
     sscanf(S, "%d (%15c", &P->pid, P->cmd);   /* comm[16] in kernel */
@@ -164,7 +170,8 @@ void stat2proc(char* S, proc_t* P) {
        "%ld "
        "%lu %lu %lu %lu %lu %lu "
        "%*s %*s %*s %*s " /* discard, no RT signals & Linux 2.1 used hex */
-       "%lu %lu %lu",
+       "%lu %lu %lu "
+       "%d %d",
        &P->state,
        &P->ppid, &P->pgrp, &P->session, &P->tty, &P->tpgid,
        &P->flags, &P->min_flt, &P->cmin_flt, &P->maj_flt, &P->cmaj_flt, &P->utime, &P->stime,
@@ -173,21 +180,18 @@ void stat2proc(char* S, proc_t* P) {
        &P->rss,
        &P->rss_rlim, &P->start_code, &P->end_code, &P->start_stack, &P->kstk_esp, &P->kstk_eip,
 /*     P->signal, P->blocked, P->sigignore, P->sigcatch,   */ /* can't use */
-       &P->wchan, &P->nswap, &P->cnswap /* , &P->exit_signal  */);
-/* TODO: add &P->exit_signal support here, perhaps to identify Linux threads */
+       &P->wchan, &P->nswap, &P->cnswap,
+/* -- Linux 2.0.35 ends here -- */
+       &P->exit_signal, &P->processor  /* 2.2.1 ends with "exit_signal" */
+/* -- Linux 2.2.8 and 2.3.47 end here -- */
+    );
     
-/*    fprintf(stderr, "stat2proc converted %d fields.\n",num); */
+    /* fprintf(stderr, "stat2proc converted %d fields.\n",num); */
     if (P->tty == 0)
 	P->tty = -1;  /* the old notty val, update elsewhere bef. moving to 0 */
-    if (linux_version_code < LINUX_VERSION(1,3,39)) {
-	P->priority = 2*15 - P->priority;	/* map old meanings to new */
-	P->nice = 15 - P->nice;
-    }
-    if (linux_version_code < LINUX_VERSION(1,1,30) && P->tty != -1)
-	P->tty = 4*0x100 + P->tty;		/* when tty wasn't full devno */
 }
 
-void statm2proc(char* s, proc_t* P) {
+static void statm2proc(char* s, proc_t* P) {
     int num;
     num = sscanf(s, "%ld %ld %ld %ld %ld %ld %ld",
 	   &P->size, &P->resident, &P->share,
@@ -195,14 +199,7 @@ void statm2proc(char* s, proc_t* P) {
 /*    fprintf(stderr, "statm2proc converted %d fields.\n",num); */
 }
 
-void nulls2sep(char* str, int len, char sep) {
-    int i;
-    for (i = 0; i < len; i++)
-    	if (str[i] == 0)
-    	    str[i] = sep;
-}
-
-int file2str(char *directory, char *what, char *ret, int cap) {
+static int file2str(char *directory, char *what, char *ret, int cap) {
     static char filename[80];
     int fd, num_read;
 
@@ -214,7 +211,7 @@ int file2str(char *directory, char *what, char *ret, int cap) {
     return num_read;
 }
 
-char** file2strvec(char* directory, char* what) {
+static char** file2strvec(char* directory, char* what) {
     char buf[2048];	/* read buf bytes at a time */
     char *p, *rbuf = 0, *endbuf, **q, **ret;
     int fd, tot = 0, n, c, end_of_file = 0;
@@ -269,9 +266,9 @@ char** file2strvec(char* directory, char* what) {
 
 
 /* These are some nice GNU C expression subscope "inline" functions.
-   The can be used with arbitrary types and evaluate their arguments
-   exactly once.
-*/
+ * The can be used with arbitrary types and evaluate their arguments
+ * exactly once.
+ */
 
 /* Test if item X of type T is present in the 0 terminated list L */
 #   define XinL(T, X, L) ( {			\
@@ -495,7 +492,7 @@ void look_up_our_self(proc_t *p) {
  */
 
 /* Traverse tree 't' breadth-first looking for a process with pid p */
-proc_t* LookupPID(proc_t* t, pid_t p) {
+static proc_t* LookupPID(proc_t* t, pid_t p) {
     proc_t* tmp = NULL;
     if (!t)
 	return NULL;
@@ -509,53 +506,6 @@ proc_t* LookupPID(proc_t* t, pid_t p) {
     return NULL;
 }
 
-proc_t* readproctree(int flags, ...) {
-    static proc_t tree;
-    PROCTAB* PT = NULL;
-    proc_t *node, *tmp=NULL, *tmp2=NULL;
-    va_list ap;
-    
-    /* pass through apropriate arguments to openproc */
-    va_start(ap, flags);
-    if (Do(UID)) {
-	/* temporary variables to ensure that va_arg() instances
-	 * are called in the right order
-	 */
-	uid_t* u;
-	int i;
-
-	u = va_arg(ap, uid_t*);
-	i = va_arg(ap, int);
-	PT = openproc(flags, u, i);
-    }
-    else if (Do(PID) || Do(TTY) || Do(STAT))
-	PT = openproc(flags, va_arg(ap, void*));
-    else
-	PT = openproc(flags);
-    va_end(ap);
-
-    /* first pass: build tree, putting orphans on the first level */
-    tree.l = tree.r = NULL;
-    while ((node = readproc(PT,0)))
-	if ((tmp = LookupPID(&tree, node->ppid))) {
-	    node->r = tmp->l->r;	/* node --> left list of parent */
-	    tmp->l->r = node;
-	} else {
-	    node->r = tree.r;		/* node --> right list of 'tree' */
-	    tree.r = node;
-	}
-    /* second pass: scan tree for PPIDs of level-1 nodes moving links as necessary */
-    for (node = &tree; node; node = node->r)
-	if ((tmp = LookupPID(&tree, node->r->ppid))) {
-	    tmp2 = node->r;			/* unlink from right list of 'tree' */
-	    node->r = node->r->r;
-	    tmp2->r = tmp->l->r;		/* insert as child of found node */
-	    tmp->l->r = node;
-	}
-    closeproc(PT);
-    return &tree;
-}
-
 /* Convenient wrapper around openproc and readproc to slurp in the whole process
  * table subset satisfying the constraints of flags and the optional PID list.
  * Free allocated memory with freeproctab().  Access via tab[N]->member.  The
@@ -591,12 +541,3 @@ proc_t** readproctab(int flags, ...) {
     closeproc(PT);
     return tab;
 }
-
-/* deallocate a table of pointers to proc structures
- */
-void freeproctab(proc_t** tab) {
-    proc_t** p;
-    for(p = tab; *p; p++)
-	freeproc(*p);
-    free(tab);
-}
