@@ -1,8 +1,11 @@
 /* Suite version information for procps utilities
  * Copyright (c) 1995 Martin Schulze <joey@infodrom.north.de>
  * Ammended by cblake to only export the function symbol.
+ * Redistributable under the terms of the
+ * GNU Library General Public License; see COPYING
  */
 #include <stdio.h>
+#include <stdlib.h>
 
 #ifdef MINORVERSION
 char procps_version[] = "procps version " VERSION "." SUBVERSION "." MINORVERSION;
@@ -23,12 +26,13 @@ void display_version(void) {
 
 int linux_version_code = 0;
 
-void set_linux_version(void) {
+static void init_Linux_version(void) __attribute__((constructor));
+static void init_Linux_version(void) {
     static struct utsname uts;
     int x = 0, y = 0, z = 0;	/* cleared in case sscanf() < 3 */
     
     if (linux_version_code) return;
-    if (uname(&uts) == -1)	/* failure most likely implies impending death */
+    if (uname(&uts) == -1)	/* failure implies impending death */
 	exit(1);
     if (sscanf(uts.release, "%d.%d.%d", &x, &y, &z) < 3)
 	fprintf(stderr,		/* *very* unlikely to happen by accident */
