@@ -25,13 +25,12 @@
 #include "proc/sysinfo.h"
 
 static char buf[128];
-double av[3];
+static double av[3];
 
 char *sprint_uptime(void) {
   struct utmp *utmpstruct;
   int upminutes, uphours, updays;
-  static int utmp_fd = -1;
-  int nread, pos;
+  int pos;
   struct tm *realtime;
   time_t realseconds;
   int numuser;
@@ -59,49 +58,20 @@ char *sprint_uptime(void) {
   uphours = uphours % 24;
   upminutes = upminutes % 60;
   if(uphours)
-    pos += sprintf(buf + pos, "%2d:%02dh, ", uphours, upminutes);
+    pos += sprintf(buf + pos, "%2d:%02d, ", uphours, upminutes);
   else
     pos += sprintf(buf + pos, "%d min, ", upminutes);
 
 /* count the number of users */
-#if 0
-  if (utmp_fd < 0) {
-    utmp_fd = open(UTMP_FILE, O_RDONLY);
-    if (utmp_fd < 0) {
-      perror("open " UTMP_FILE);
-      exit(1);
-    }
-  }
 
-  lseek(utmp_fd, 0L, SEEK_SET);
-
-  numuser = 0;
-  while (1) {
-    nread = read(utmp_fd, &utmpstruct, sizeof utmpstruct);
-    if (nread == 0)
-      break;
-    if (nread < 0) {
-      perror("read " UTMP_FILE);
-      exit(1);
-    }
-    if (nread != sizeof utmpstruct) {
-      fprintf(stderr, "bad data in " UTMP_FILE);
-      exit(1);
-    }
-    if ((utmpstruct.ut_type == USER_PROCESS) &&
-	(utmpstruct.ut_name[0] != '\0'))
-      numuser++;
-  }
-#else
   numuser = 0;
   setutent();
-  while ((utmpstruct=getutent())!= NULL ) {
-    if ((utmpstruct->ut_type == USER_PROCESS) && (utmpstruct->ut_name[0] != '\0'))
+  while ((utmpstruct = getutent())) {
+    if ((utmpstruct->ut_type == USER_PROCESS) &&
+       (utmpstruct->ut_name[0] != '\0'))
       numuser++;
   }
   endutent();
-#endif 
-  
 
   pos += sprintf(buf + pos, "%2d user%s, ", numuser, numuser == 1 ? "" : "s");
 
