@@ -3,7 +3,7 @@
 #ifndef __WHATTIME_H
 #define __WHATTIME_H
 
-void print_uptime(void);
-char *sprint_uptime(void);
+extern void print_uptime(void);
+extern char *sprint_uptime(void);
 
 #endif
