@@ -41,16 +41,17 @@
 #define SEL_COMM 13
 
 /* Since an enum could be smashed by a #define, it would be bad. */
-#define U98 0 /* Unix98 standard */    /* This must be 0 */
-#define XXX 1 /* Common extension */
-#define DEC 2 /* Digital Unix */
-#define AIX 3 /* AIX */
-#define SCO 4 /* SCO */
-#define LNX 5 /* Linux original :-) */
-#define BSD 6 /* FreeBSD and OpenBSD */
-#define SUN 7 /* SunOS 5 (Solaris) */
-#define HPU 8 /* HP-UX */
-#define SGI 9 /* Irix */
+#define U98  0 /* Unix98 standard */    /* This must be 0 */
+#define XXX  1 /* Common extension */
+#define DEC  2 /* Digital Unix */
+#define AIX  3 /* AIX */
+#define SCO  4 /* SCO */
+#define LNX  5 /* Linux original :-) */
+#define BSD  6 /* FreeBSD and OpenBSD */
+#define SUN  7 /* SunOS 5 (Solaris) */
+#define HPU  8 /* HP-UX */
+#define SGI  9 /* Irix */
+#define SOE 10 /* IBM's S/390 OpenEdition */
 
 /*
  * Must not overflow the output buffer:
@@ -81,15 +82,12 @@
 #define PER_FORCE_BSD     0x0010
 #define PER_GOOD_o        0x0020
 #define PER_OLD_m         0x0040
-#define PER_SUN_MUTATE_a  0x0080
+#define PER_NO_DEFAULT_g  0x0080
 #define PER_ZAP_ADDR      0x0100
 #define PER_SANE_USER     0x0200
 #define PER_IRIX_l        0x0400
 
-/* Simple selections by bit mask
- * Warning: these bit positions are hard-coded into the table in
- * table_accept(), so do not change them.
- */
+/* Simple selections by bit mask */
 #define SS_B_x 0x01
 #define SS_B_g 0x02
 #define SS_U_d 0x04
@@ -115,6 +113,8 @@
 #define FM_L 0x0008 /* -L */
 #define FM_P 0x0010 /* -P */
 #define FM_M 0x0020 /* -M */
+#define FM_T 0x0040 /* -T */
+#define FM_F 0x0080 /* -F */  /* -F also sets the regular -f flags */
 
 /* sorting & formatting */
 /* U,B,G is Unix,BSD,Gnu and then there is the option itself */
@@ -224,17 +224,30 @@ extern int escape_strlist(char *dst, const char **src, size_t n);
 extern int escape_str(char *dst, const char *src, size_t n);
 extern int octal_escape_str(char *dst, const char *src, size_t n);
 extern int simple_escape_str(char *dst, const char *src, size_t n);
-extern int html_escape_str(char *dst, const char *src, size_t n);
 
 /********************* UNDECIDED GLOBALS **************/
 
-/* compare.c */
-extern void reset_sort_options(void);
-extern int mult_lvl_cmp(void* a, void* b);
-extern int node_mult_lvl_cmp(void* a, void* b);
-extern void dump_keys(void);
-extern const char *parse_sort_opt(const char* opt);
-extern const char *parse_long_sort(const char* opt);
+/*
+ * fputs(3) should (as in "good behavior") return the number of
+ * characters written as it does on Digital Unix, AIX, Irix, and SunOS.
+ * I'll assume glibc 2.1 has this extremely useful feature.
+ *
+ * Note: code ported from other systems will keep breaking until
+ * the library is updated. You should patch the library itself if
+ * at all possible. (for example, distributers who build libc from
+ * source with automatic patching as part of the build process)
+ */
+
+#if defined __GLIBC__ && ((__GLIBC__ == 2 && __GLIBC_MINOR__ > 2) || __GLIBC__ > 2)
+#warning Hopefully fputs(3) has been modernized...
+#else
+#define EMULATE_FPUTS
+#define fputs something_to_avoid_libc_troubles
+static inline int fputs(const char *s, FILE *fp){
+  return fwrite(s,1,strlen(s),fp);
+  /* return fprintf(fp, "%s", s); */
+}
+#endif
 
 /* output.c */
 extern void show_one_proc(proc_t* p);
@@ -258,7 +271,6 @@ extern char           *bsd_v_format;
 extern int             bsd_c_option;
 extern int             bsd_e_option;
 extern uid_t           cached_euid;
-extern pid_t           cached_pid;
 extern dev_t           cached_tty;
 extern char            forest_prefix[4 * 32*1024 + 100];
 extern int             forest_type;
@@ -273,10 +285,11 @@ extern int             max_line_width;
 extern const char     *namelist_file;
 extern int             negate_selection;
 extern unsigned        personality;
-extern int             prefer_bsd;
+extern int             prefer_bsd_defaults;
 extern int             running_only;
 extern int             screen_cols;
 extern int             screen_rows;
+extern unsigned long   seconds_since_boot;
 extern selection_node *selection_list;
 extern unsigned        simple_select;
 extern sort_node      *sort_list;
@@ -285,7 +298,6 @@ extern char           *sysv_fl_format;
 extern char           *sysv_j_format;
 extern char           *sysv_l_format;
 extern int             unix_f_option;
-extern int             use_aix_codes;
 extern int             user_is_number;
 extern int             wchan_is_number;
 
@@ -309,7 +321,4 @@ extern void self_info(void);
 /* parser.c */
 extern int arg_parse(int argc, char *argv[]);
 
-/* output.c */
-extern unsigned long seconds_since_boot;
-
 #endif
