@@ -50,7 +50,6 @@ char           *bsd_v_format = (char *)0xdeadbeef;
 int             bsd_c_option = -1;
 int             bsd_e_option = -1;
 uid_t           cached_euid = -1;
-pid_t           cached_pid = -1;
 dev_t           cached_tty = -1;
 char            forest_prefix[4 * 32*1024 + 100];
 int             forest_type = -1;
@@ -61,14 +60,14 @@ int             header_gap = -1;
 int             header_type = -1;
 int             include_dead_children = -1;
 int             lines_to_next_header = -1;
-int             max_line_width = 666;
 const char     *namelist_file = (const char *)0xdeadbeef;
 int             negate_selection = -1;
 int             running_only = -1;
 unsigned        personality = 0xffffffff;
-int             prefer_bsd = -1;
+int             prefer_bsd_defaults = -1;
 int             screen_cols = -1;
 int             screen_rows = -1;
+unsigned long   seconds_since_boot = -1;
 selection_node *selection_list = (selection_node *)0xdeadbeef;
 unsigned        simple_select = 0xffffffff;
 sort_node      *sort_list = (sort_node *)0xdeadbeef; /* ready-to-use sort list */
@@ -77,7 +76,6 @@ char           *sysv_fl_format = (char *)0xdeadbeef;
 char           *sysv_j_format = (char *)0xdeadbeef;
 char           *sysv_l_format = (char *)0xdeadbeef;
 int             unix_f_option = -1;
-int             use_aix_codes = -1;
 int             user_is_number = -1;
 int             wchan_is_number = -1;
 
@@ -98,6 +96,12 @@ static void reset_selection_list(void){
   selection_list = NULL;
 }
 
+/* The rules:
+ * 1. Defaults are implementation-specific. (ioctl,termcap,guess)
+ * 2. COLUMNS and LINES override the defaults. (standards compliance)
+ * 3. Command line options override everything else.
+ * 4. Actual output may be more if the above is too narrow.
+ */
 static void set_screen_size(void){
   struct winsize ws;
   char *columns; /* Unix98 environment variable */
@@ -109,6 +113,7 @@ static void set_screen_size(void){
     screen_cols = 80;
     screen_rows = 24;
   }
+  if(!isatty(STDOUT_FILENO)) screen_cols = OUTBUF_SIZE;
   columns = getenv("COLUMNS");
   if(columns && *columns){
     long t;
@@ -131,7 +136,7 @@ static void set_screen_size(void){
 
 /**************** personality control **************/
 
-typedef struct braindead_c_syntax {
+typedef struct personality_table_struct {
   const char *name; /* personality name */
   const void *jump; /* See gcc extension info.   :-)   */
 } personality_table_struct;
@@ -147,6 +152,7 @@ static const char *set_personality(void){
   personality_table_struct findme = { buf, NULL};
   personality_table_struct *found;
   static const personality_table_struct personality_table[] = {
+  {"390",      &&case_390},
   {"aix",      &&case_aix},
   {"bsd",      &&case_bsd},
   {"compaq",   &&case_compaq},
@@ -159,12 +165,15 @@ static const char *set_personality(void){
   {"irix",     &&case_irix},
   {"linux",    &&case_linux},
   {"old",      &&case_old},
+  {"os390",    &&case_os390},
   {"posix",    &&case_posix},
+  {"s390",     &&case_s390},
   {"sco",      &&case_sco},
   {"sgi",      &&case_sgi},
   {"solaris2", &&case_solaris2},
   {"sunos4",   &&case_sunos4},
   {"sysv",     &&case_sysv},
+  {"tru64",    &&case_tru64},
   {"unix",     &&case_unix},
   {"unix95",   &&case_unix95},
   {"unix98",   &&case_unix98},
@@ -173,7 +182,7 @@ static const char *set_personality(void){
   const int personality_table_count = sizeof(personality_table)/sizeof(personality_table_struct);
 
   personality = 0;
-  prefer_bsd = 0;
+  prefer_bsd_defaults = 0;
 
   bsd_j_format = "OL_j";
   bsd_l_format = "OL_l";
@@ -204,11 +213,10 @@ static const char *set_personality(void){
   if(!found) return "Environment specified an unknown personality.";
 
   goto *(found->jump);    /* See gcc extension info.  :-)   */
-			/* FIXME: change to enum's and switch statement */
 
   case_bsd:
     personality = PER_FORCE_BSD | PER_BSD_h | PER_BSD_m;
-    prefer_bsd = 1;
+    prefer_bsd_defaults = 1;
     bsd_j_format = "FB_j";
     bsd_l_format = "FB_l";
     /* bsd_s_format not used */
@@ -218,13 +226,13 @@ static const char *set_personality(void){
 
   case_old:
     personality = PER_FORCE_BSD | PER_OLD_m;
-    prefer_bsd = 1;
+    prefer_bsd_defaults = 1;
     return NULL;
 
   case_debian:  /* Toss this? They don't seem to care much. */
   case_gnu:
-    personality = PER_GOOD_o | PER_CUMUL_MARKED;
-    prefer_bsd = 1;
+    personality = PER_GOOD_o | PER_CUMUL_MARKED | PER_OLD_m;
+    prefer_bsd_defaults = 1;
     sysv_f_format  = "RD_f";
     /* sysv_fl_format = "RD_fl"; */   /* Debian can't do this! */
     sysv_j_format  = "RD_j";
@@ -254,10 +262,11 @@ static const char *set_personality(void){
     bsd_v_format = "FB_v";
     return NULL;
 
+  case_tru64:
   case_compaq:
   case_digital:
     personality = PER_GOOD_o | PER_BSD_h;
-    prefer_bsd = 1;   /* TODO check this */
+    prefer_bsd_defaults = 1;
     sysv_f_format  = "F5FMT";
     sysv_fl_format = "FL5FMT";
     sysv_j_format  = "JFMT";
@@ -270,8 +279,8 @@ static const char *set_personality(void){
     return NULL;
 
   case_sunos4:
-    personality = PER_SUN_MUTATE_a;
-    prefer_bsd = 1;
+    personality = PER_NO_DEFAULT_g;
+    prefer_bsd_defaults = 1;
     bsd_j_format = "FB_j";
     bsd_l_format = "FB_l";
     /* bsd_s_format not used */
@@ -286,6 +295,12 @@ static const char *set_personality(void){
     else personality = PER_IRIX_l;
     return NULL;
 
+  case_os390:  /* IBM's OS/390 OpenEdition on the S/390 mainframe */
+  case_s390:
+  case_390:
+    sysv_j_format  = "J390";  /* don't know what -jl and -jf do */
+    return NULL;
+
   case_hp:
   case_hpux:
   case_posix:
@@ -312,7 +327,6 @@ void reset_global(void){
   bsd_c_option          = 0;
   bsd_e_option          = 0;
   cached_euid           = geteuid();
-  cached_pid            = getpid();
   cached_tty            = p.tty;
 /* forest_prefix must be all zero because of POSIX */
   forest_type           = 0;
@@ -323,21 +337,23 @@ void reset_global(void){
   header_type           = HEAD_SINGLE;
   include_dead_children = 0;
   lines_to_next_header  = 1;
-  max_line_width        = 80;
   namelist_file         = NULL;
   negate_selection      = 0;
   running_only          = 0;
+  seconds_since_boot    = uptime(0,0);
   selection_list        = NULL;
   simple_select         = 0;
   sort_list             = NULL;
   unix_f_option         = 0;
-  use_aix_codes         = 0;
   user_is_number        = 0;
   wchan_is_number       = 0;
 }
 
 /*********** spew variables ***********/
 void self_info(void){
+#ifndef EMULATE_FPUTS
+  int count;
+#endif
   fprintf(stderr,
     "BSD j    %s\n"
     "BSD l    %s\n"
@@ -371,11 +387,25 @@ void self_info(void){
     __GNU_LIBRARY__, __GLIBC__, __GLIBC_MINOR__
   );
 
+#ifdef EMULATE_FPUTS
+  fprintf(stderr, "libc assumed lame, using fprintf to emulate fputs.\n\n");
+#else
+  fprintf(stderr, "fputs(\"");
+  count = fputs("123456789", stderr);
+  fprintf(stderr, "\", stderr) gives %d, which is %s.\n",
+    count, count==9?"good":"BAD!\nAdjust ps/common.h or libc, then recompile"
+  );
+  if(count!=9){
+    fprintf(stderr, "(procps includes a libc patch called glibc.patch)\n");
+  }
+  fprintf(stderr, "\n");
+#endif
+
   fprintf(stderr,
-    "header_gap=%d lines_to_next_header=%d max_line_width=%d\n"
+    "header_gap=%d lines_to_next_header=%d\n"
     "screen_cols=%d screen_rows=%d\n"
     "\n",
-    header_gap, lines_to_next_header, max_line_width,
+    header_gap, lines_to_next_header,
     screen_cols, screen_rows
   );
 
