@@ -76,24 +76,14 @@ static char *whitespace = whitespace_and_outbuf;
 static unsigned max_rightward = 0x12345678; /* space for RIGHT stuff */
 static unsigned max_leftward = 0x12345678; /* space for LEFT stuff */
 
-/* Justification control for flags field. */
-#define JUST_MASK 0x0f
-     /* AIXHACK   0 */
-#define USER      1  /* left if text, right if numeric */
-#define LEFT      2
-#define RIGHT     3
-#define UNLIMITED 4
-#define WCHAN     5  /* left if text, right if numeric */
-#define SIGNAL    6  /* right in 9, or 16 if screen_cols>107 */
-
-#define CUMUL     16  /* mark cumulative (Summed) headers with 'C' */
-
 static int wide_signals;  /* true if we have room */
 
 static proc_t *pp;     /* the process being printed */
 
+unsigned long seconds_since_boot;
 static unsigned long seconds_since_1970;
 static unsigned long time_of_boot;
+static unsigned long bytes_main_memory;
 static unsigned long page_shift;
 
 
@@ -376,7 +366,7 @@ static int pr_fname(void){
 /* elapsed wall clock time, [[dd-]hh:]mm:ss format (not same as "time") */
 static int pr_etime(void){
   unsigned t, dd,hh,mm,ss;
-  char *cp = outbuf;
+  int c = 0;
   t = (
           ((unsigned long)seconds_since_boot)
         - ((unsigned long)pp->start_time)
@@ -389,10 +379,10 @@ static int pr_etime(void){
   hh = t%24;
   t /= 24;
   dd = t;
-  cp +=(     dd      ?  sprintf(cp, "%u-", dd)           :  0 );
-  cp +=( (dd || hh)  ?  sprintf(cp, "%02u:", hh)         :  0 );
-  cp +=                 sprintf(cp, "%02u:%02u", mm, ss)       ;
-  return (int)(cp-outbuf);
+  c +=(      dd      ?  sprintf(outbuf, "%u-", dd)    :  sprintf(outbuf, "   ")   );
+  c +=(  (dd || hh)  ?  sprintf(outbuf+c, "%02u:", hh)  :  sprintf(outbuf+c, "   ")   );
+  c +=(                 sprintf(outbuf+c, "%02u:%02u", mm, ss)             );
+  return c;
 }
 static int pr_nice(void){
   return sprintf(outbuf, "%ld", pp->nice);
@@ -400,50 +390,12 @@ static int pr_nice(void){
 
 /* "Processor utilisation for scheduling."  --- we use %cpu w/o fraction */
 static int pr_c(void){
-  unsigned long total_time;   /* jiffies used by this process */
-  unsigned long pcpu = 0;     /* scaled %cpu, 999 means 99.9% */
-  unsigned long seconds;      /* seconds of process life */
-  total_time = pp->utime + pp->stime;
-  if(include_dead_children) total_time += (pp->cutime + pp->cstime);
-  seconds =
-    seconds_since_boot  -  ((unsigned long)pp->start_time)  /  Hertz
-  ;
-  /* Use 100ULL (not 100) to avoid 32-bit overflow. */
-  if(seconds) pcpu = (total_time * 100ULL / Hertz) / seconds;
-  if (pcpu > 99) pcpu = 99;
-  return sprintf(outbuf, "%2u", (unsigned)pcpu);
+  return sprintf(outbuf, "%2u", (unsigned)(pp->pcpu / 10));
 }
-/* normal %CPU in ##.# format. */
 static int pr_pcpu(void){
-  unsigned long total_time;   /* jiffies used by this process */
-  unsigned long pcpu = 0;     /* scaled %cpu, 999 means 99.9% */
-  unsigned long seconds;      /* seconds of process life */
-  total_time = pp->utime + pp->stime;
-  if(include_dead_children) total_time += (pp->cutime + pp->cstime);
-  seconds =
-    seconds_since_boot  -  ((unsigned long)pp->start_time)  /  Hertz
-  ;
-  /* Use 1000ULL (not 1000) to avoid 32-bit overflow. */
-  if(seconds) pcpu = (total_time * 1000ULL / Hertz) / seconds;
-  if (pcpu > 999) pcpu = 999;
-  return sprintf(outbuf, "%2u.%u", (unsigned)(pcpu/10), (unsigned)(pcpu%10));
+  return sprintf(outbuf, "%2u.%u",
+  		(unsigned)(pp->pcpu / 10), (unsigned)(pp->pcpu % 10));
 }
-/* this is a "per-mill" format, like %cpu with no decimal point */
-static int pr_cp(void){
-  unsigned long total_time;   /* jiffies used by this process */
-  unsigned long pcpu = 0;     /* scaled %cpu, 999 means 99.9% */
-  unsigned long seconds;      /* seconds of process life */
-  total_time = pp->utime + pp->stime;
-  if(include_dead_children) total_time += (pp->cutime + pp->cstime);
-  seconds =
-    seconds_since_boot  -  ((unsigned long)pp->start_time)  /  Hertz
-  ;
-  /* Use 1000ULL (not 1000) to avoid 32-bit overflow. */
-  if(seconds) pcpu = (total_time * 1000ULL / Hertz) / seconds;
-  if (pcpu > 999) pcpu = 999;
-  return sprintf(outbuf, "%3u", (unsigned)pcpu);
-}
-
 static int pr_pgid(void){
   return sprintf(outbuf, "%u", pp->pgrp);
 }
@@ -457,17 +409,10 @@ static int pr_ppid(void){
 
 /* cumulative CPU time, [dd-]hh:mm:ss format (not same as "etime") */
 static int pr_time(void){
-  unsigned t, dd,hh,mm,ss;
-  int c;
-  t = (unsigned)(
-      (
-        (unsigned long)(pp->utime)
-        +
-        (unsigned long)(pp->stime)
-      )
-      /
-      (unsigned long)Hertz
-  );
+  int t, dd,hh,mm,ss;
+  int c = 0;
+  t = pp->utime + pp->stime;
+  t /= Hertz;
   ss = t%60;
   t /= 60;
   mm = t%60;
@@ -475,8 +420,8 @@ static int pr_time(void){
   hh = t%24;
   t /= 24;
   dd = t;
-  c  =( dd ? sprintf(outbuf, "%u-", dd) : 0              );
-  c +=( sprintf(outbuf+c, "%02u:%02u:%02u", hh, mm, ss)    );
+  c +=( dd ? sprintf(outbuf, "%d-", dd) : 0              );
+  c +=( sprintf(outbuf+c, "%02d:%02d:%02d", hh, mm, ss)    );
   return c;
 }
 
@@ -492,7 +437,7 @@ static int pr_vsz(void){
  */
 
 static int pr_ruser(void){
-  if(user_is_number || (strlen(pp->euser)>max_rightward))
+  if(user_is_number || (strlen(pp->ruser)>max_rightward))
     return sprintf(outbuf, "%d", pp->ruid);
   return sprintf(outbuf, "%s", pp->ruser);
 }
@@ -735,10 +680,9 @@ static int pr_rss(void){
   return sprintf(outbuf, "%lu", pp->rss << (page_shift - 10));
 }
 
-/* On 32-bit hardware, this handles at least 8 GB. Use 1000ULL for more. */
 static int pr_pmem(void){
   unsigned long pmem = 0;
-  pmem = pp->rss * 1000 / (kb_main_total >> (page_shift-10));
+  pmem = pp->rss * 1000 / (bytes_main_memory >> page_shift);
   return sprintf(outbuf, "%2u.%u", (unsigned)(pmem/10), (unsigned)(pmem%10));
 }
 
@@ -932,7 +876,7 @@ static int pr_sgi_p(void){          /* FIXME */
 /* temporary hack -- mark new stuff grabbed from Debian ps */
 #define LNx LNX
 
-/* there are about 195 listed */
+/* there are about 194 listed */
 
 /* Many of these are placeholders for unsupported options. */
 static const format_struct format_array[] = {
@@ -942,7 +886,6 @@ static const format_struct format_array[] = {
 {"acflag",    "ACFLG",   pr_nop,      sr_nop,     5,   0,    XXX, RIGHT}, /*acflg*/
 {"acflg",     "ACFLG",   pr_nop,      sr_nop,     5,   0,    BSD, RIGHT}, /*acflag*/
 {"addr",      "ADDR",    pr_nop,      sr_nop,     4,   0,    XXX, RIGHT},
-{"addr_1",    "ADDR",    pr_nop,      sr_nop,     1,   0,    LNX, LEFT},
 {"alarm",     "ALARM",   pr_alarm,    sr_it_real_value, 5, 0, LNX, RIGHT},
 {"argc",      "ARGC",    pr_nop,      sr_nop,     4,   0,    LNX, RIGHT},
 {"args",      "COMMAND", pr_args,     sr_nop,    16,   0,    U98, UNLIMITED}, /*command*/
@@ -960,7 +903,7 @@ static const format_struct format_array[] = {
 {"cnswap",    "-",       pr_nop,      sr_cnswap,  1,   0,    LNX, RIGHT},
 {"comm",      "COMMAND", pr_comm,     sr_nop,    16,   0,    U98, UNLIMITED}, /*ucomm*/
 {"command",   "COMMAND", pr_args,     sr_nop,    16,   0,    XXX, UNLIMITED}, /*args*/
-{"cp",        "CP",      pr_cp,       sr_pcpu,    3,   0,    DEC, RIGHT}, /*cpu*/
+{"cp",        "CP",      pr_nop,      sr_nop,     2,   0,    DEC, RIGHT}, /*cpu*/
 {"cpu",       "CPU",     pr_nop,      sr_nop,     3,   0,    BSD, RIGHT}, /* FIXME ... HP-UX wants this as the CPU number for SMP? */
 {"cputime",   "TIME",    pr_time,     sr_nop,     8,   0,    DEC, RIGHT}, /*time*/
 {"cstime",    "-",       pr_nop,      sr_cstime,  1,   0,    LNX, RIGHT},
@@ -1151,7 +1094,7 @@ static const int format_array_count = sizeof(format_array)/sizeof(format_struct)
 /* First X field may be NR, which is p->start_code>>26 printed with %2ld */
 /* That seems useless though, and Debian already killed it. */
 /* The ones marked "Digital" have the name defined, not just the data. */
-static const macro_struct macro_array[] = {
+const macro_struct macro_array[] = {
 {"DFMT",     "pid,tname,state,cputime,cmd"},         /* Digital's default */
 {"DefBSD",   "pid,tname,stat,bsdtime,args"},               /* Our BSD default */
 {"DefSysV",  "pid,tname,time,cmd"},                     /* Our SysV default */
@@ -1212,12 +1155,12 @@ static const macro_struct macro_array[] = {
 {"~", "~"} /* NULL would ruin alphabetical order */
 };
 
-static const int macro_array_count = sizeof(macro_array)/sizeof(macro_struct);
+const int macro_array_count = sizeof(macro_array)/sizeof(macro_struct);
 
 
 /*************************** AIX formats ********************/
 /* Convert AIX format codes to normal format specifiers. */
-static const aix_struct aix_array[] = {
+const aix_struct aix_array[] = {
 {'C', "pcpu",   "%CPU"},
 {'G', "group",  "GROUP"},
 {'P', "ppid",   "PPID"},
@@ -1235,12 +1178,12 @@ static const aix_struct aix_array[] = {
 {'z', "vsz",    "VSZ"},
 {'~', "~",      "~"} /* NULL would ruin alphabetical order */
 };
-static const int aix_array_count = sizeof(aix_array)/sizeof(aix_struct);
+const int aix_array_count = sizeof(aix_array)/sizeof(aix_struct);
 
 
 /********************* sorting ***************************/
 /* Convert short sorting codes to normal format specifiers. */
-static const shortsort_struct shortsort_array[] = {
+const shortsort_struct shortsort_array[] = {
 {'C', "pcpu"       },
 {'G', "tpgid"      },
 {'J', "cstime"     },
@@ -1269,7 +1212,7 @@ static const shortsort_struct shortsort_array[] = {
 {'y', "priority"   }, /* nice */
 {'~', "~"          } /* NULL would ruin alphabetical order */
 };
-static const int shortsort_array_count = sizeof(shortsort_array)/sizeof(shortsort_struct);
+const int shortsort_array_count = sizeof(shortsort_array)/sizeof(shortsort_struct);
 
 
 /*********** print format_array **********/
@@ -1501,7 +1444,7 @@ void show_one_proc(proc_t* p){
     if(space>SPACE_AMOUNT) space=SPACE_AMOUNT;
 /* printf("[%d %d %d %d %d %d]", correct,actual,amount,leftpad,space,dospace); */
     /* print data, set x position stuff */
-    actual += fputs(outbuf-space, stdout);
+    actual += fprintf(stdout, "%s", outbuf-space);
     correct += fmt->width;
     correct += legit;        /* adjust for SIGNAL expansion */
     if(fmt->pr && fmt->next && fmt->next->pr){ /* neither is AIX filler */
@@ -1531,9 +1474,10 @@ void init_output(void){
     PAGE_SIZE, PROT_NONE
   );
 #endif
+  seconds_since_boot = uptime(0,0);
   seconds_since_1970 = time(NULL);
   time_of_boot = seconds_since_1970 - seconds_since_boot;
-  meminfo();
+  bytes_main_memory = read_total_main();
   switch(getpagesize()){
   case 65536: page_shift = 16; break;
   case 32768: page_shift = 15; break;
