@@ -79,7 +79,7 @@ static const char *parse_pid(char *str, sel_union *ret){
   num = strtol(str, &endp, 0);
   if(*endp != '\0')   return pidsyntax;
   if(num>0x7fff)      return pidrange;  /* Linux PID limit */
-  if(num<1)           return pidrange;
+  if(num<0)           return pidrange;
   ret->pid = num;
   return 0;
 }
@@ -289,12 +289,6 @@ static const char *parse_sysv_option(void){
       trace("-R selects PRM groups\n");
       return "Don't understand PRM on Linux.";
       break;
-    case 'T':     /* unknown IRIX feature */
-      /* IRIX 6.5 docs suggest POSIX threads get shown individually. */
-      /* This would make -T be like -L, -m, and m. (but an extra column) */
-      trace("-T adds strange SPID column (old sproc() threads?)\n");
-      return "Please tell the author what -T is for.";
-      break;
 #endif
     case 'U': /* end */
       trace("-U select by RUID (supports names).\n");
@@ -439,9 +433,11 @@ static const char *parse_sysv_option(void){
       return "Don't understand MAC on Linux.";
       break;
     case '-':
+      printf("ARRRGH!!! -\n");
       return "Embedded '-' among SysV options makes no sense.";
       break;
     case '\0':
+      printf("ARRRGH!!! \\0\n");
       return "Please report the \"SysV \\0 can't happen\" bug.";
       break;
     default:
@@ -456,6 +452,14 @@ static const char *parse_bsd_option(void){
   const char *arg;
   const char *err;
 
+#if 0
+  fprintf(stderr, "(%s)   %sforce_bsd   %s(personality & PER_FORCE_BSD)\n",
+    ps_argv[thisarg],
+    (force_bsd?"":"!"),
+    ((personality & PER_FORCE_BSD)?"":"!")
+  );
+#endif
+
   flagptr = ps_argv[thisarg];  /* assume we _have_ a '-' */
   if(flagptr[0]=='-'){
     if(!force_bsd) return "Can't happen!  Problem #1.";
@@ -470,7 +474,6 @@ static const char *parse_bsd_option(void){
 
   while(*++flagptr){
     switch(*flagptr){
-#if 0
     case 'A':
       /* maybe this just does a larger malloc() ? */
       trace("A Increases the argument space (Digital Unix)\n");
@@ -481,18 +484,15 @@ static const char *parse_bsd_option(void){
       trace("C Use raw CPU time for %%CPU instead of decaying ave\n");
       return "Option C is reserved.";
       break;
-#endif
     case 'L': /* single */
       trace("L List all format specifiers\n");
       exclusive("L");
       print_format_specifiers();
       exit(0);
-#if 0
     case 'M':
       trace("M junk (use alternate core)\n");
       return "Option M is unsupported, try N or -n instead.";
       break;
-#endif
     case 'N': /* end */
       trace("N Specify namelist file\n");
       arg=get_opt_arg();
@@ -547,7 +547,9 @@ static const char *parse_bsd_option(void){
       break;
     case 'a':
       trace("a Select all w/tty, including other users\n");
-      simple_select |= SS_B_a;
+      /* Now the PER_SUN_MUTATE_a flag is handled elsewhere. */
+      /* if(personality & PER_SUN_MUTATE_a) simple_select |= SS_U_a;
+      else               */              simple_select |= SS_B_a;
       break;
     case 'c':
       trace("c true command name\n");
@@ -575,12 +577,10 @@ static const char *parse_bsd_option(void){
       trace("j job control format\n");
       format_flags |= FF_Bj;
       break;
-#if 0
     case 'k':
       trace("k N/A Use /vmcore as c-dumpfile\n");
       return "Obsolete k option not supported.";
       break;
-#endif
     case 'l':
       trace("l Display long format\n");
       format_flags |= FF_Bl;
@@ -595,10 +595,10 @@ static const char *parse_bsd_option(void){
         defer_sf_option("pmem", SF_B_m);
         break;
       }
-      return "Thread support not implemented.";
+      return "Thread display not implemented.";
       break;
     case 'n':
-      trace("n Numeric output for WCHAN, and USER replaced by UID\n");
+      trace("n Numeric output for WCHAN and USER replaced by UID\n");
       wchan_is_number = 1;
       user_is_number = 1;
       /* TODO add tty_is_number too? */
@@ -662,13 +662,32 @@ static const char *parse_bsd_option(void){
       simple_select |= SS_B_x;
       break;
     case '-':
+      printf("ARRRGH!!! -\n");
       return "Embedded '-' among BSD options makes no sense.";
       break;
     case '\0':
+      printf("ARRRGH!!! \\0\n");
       return "Please report the \"BSD \\0 can't happen\" bug.";
       break;
     default:
-      return "Unsupported option (BSD syntax)";
+      if (*flagptr >= '0' && *flagptr <= '9') {
+        /* BSD syntax allows (in fact, on some systems requires)
+	 * being able to include the pid after the options with
+	 * NO intervening space.  :-(
+	 */
+        selection_node *pidnode;
+
+        pidnode = calloc(1, sizeof(selection_node));
+	pidnode->u = calloc(1, sizeof(sel_union));
+	pidnode->n = 1;
+        parse_pid(flagptr, pidnode->u);
+	pidnode->next = selection_list;
+	selection_list = pidnode;
+	selection_list->typecode = SEL_PID;
+	return NULL; /* do not iterate over the rest of the digits */
+      } else {
+        return "Unsupported option (BSD syntax)";
+      }
     } /* switch */
   } /* while */
   return NULL;
@@ -852,7 +871,7 @@ static const char *parse_gnu_option(void){
   case_help:
     trace("--help\n");
     exclusive("--help");
-    fputs(help_message, stderr);
+    fputs(help_message, stdout);
     exit(0);
     return NULL;
   case_info:
@@ -1035,13 +1054,13 @@ static const char *parse_all_options(void){
     case ARG_BSD:
         if(force_bsd && !(personality & PER_FORCE_BSD)) return "way bad";
       }
-      prefer_bsd_defaults = 1;
+      prefer_bsd = 1;
       err = parse_bsd_option();
       break;
     case ARG_PGRP:
     case ARG_SESS:
     case ARG_PID:
-      prefer_bsd_defaults = 1;
+      prefer_bsd = 1;
       err = parse_trailing_pids();
       break;
     case ARG_END:
@@ -1094,6 +1113,7 @@ try_bsd:
   reset_global();
   reset_parser();
   reset_sortformat();
+  reset_sort_options();
   format_flags = 0;
   ps_argc = argc;
   ps_argv = argv;
@@ -1101,10 +1121,6 @@ try_bsd:
   /* no need to reset flagptr */
   not_pure_unix=1;
   force_bsd=1;
-  prefer_bsd_defaults=1;
-  if(!( (PER_OLD_m|PER_BSD_m) & personality )) /* if default m setting... */
-    personality |= PER_OLD_m; /* Prefer old Linux over true BSD. */
-  /* Do not set PER_FORCE_BSD! It is tested below. */
 
   err2 = parse_all_options();
   if(err2) goto total_failure;
@@ -1113,17 +1129,20 @@ try_bsd:
   err2 = select_bits_setup();
   if(err2) goto total_failure;
 
+#if 0
+  /* I promised people that this would go away with the new version... */
   if(!(personality & PER_FORCE_BSD))
     fprintf(stderr, "Bad syntax, perhaps a bogus '-'?\n");
+#endif
 
   choose_dimensions();
   return 0;
 
 total_failure:
   reset_parser();
-  if(personality & PER_FORCE_BSD) fprintf(stderr, "ERROR: %s\n", err2);
-  else fprintf(stderr, "ERROR: %s\n", err);
-  fputs(help_message, stderr);
+  if(personality & PER_FORCE_BSD) fprintf(stderr, "ps: error: %s\n", err2);
+  else fprintf(stderr, "ps: error: %s\n", err);
+  fputs(usage_message, stderr);
   exit(1);
   /* return 1; */ /* useless */
 }
