@@ -82,12 +82,11 @@ static format_node *do_one_spec(const char *spec, const char *override){
     }
     return list;
   }
-
   return NULL;   /* bad, spec not found */
 }
 
 
-/* must wrap user format in default */
+/************ must wrap user format in default *************/
 static void O_wrap(sf_node *sfn, int otype){
   format_node *fnode;
   format_node *endp;
@@ -107,95 +106,7 @@ static void O_wrap(sf_node *sfn, int otype){
   sfn->f_cooked = fnode;
 }
 
-/*
- * Used to parse option O lists. Option O is shared between
- * sorting and formatting. Users may expect one or the other.
- * The "broken" flag enables a really bad Unix98 misfeature.
- * Put each completed format_node onto the list starting at ->f_cooked
- */
-static const char *format_parse(sf_node *sfn){
-  char *buf;                   /* temp copy of arg to hack on */
-  char *sep_loc;               /* separator location: " \t,\n" */
-  char *walk;
-  const char *err;       /* error code that could or did happen */
-  format_node *fnode;
-  int items;
-  int need_item;
-
-  /*** prepare to operate ***/
-  buf = malloc(strlen(sfn->sf)+1);
-  strcpy(buf, sfn->sf);
-  
-  /*** sanity check and count items ***/
-  need_item = 1; /* true */
-  items = 0;
-  walk = buf;
-  err = "Improper format specifier list.";
-  do{
-    switch(*walk){
-    case ' ': case ',': case '\t': case '\n': case '\0':
-    /* Linux extension: allow \t and \n as delimiters */
-      if(need_item){
-        free(buf);
-        return "Improper format list";
-      }
-      need_item=1;
-      break;
-    case '=':
-      if(broken) goto out;
-      /* fall through */
-    default:
-      if(need_item) items++;
-      need_item=0;
-    }
-  } while (*++walk);
-out:
-  if(!items){
-    free(buf);
-    return "Empty format list.";
-  }
-#ifdef STRICT_LIST
-  if(need_item){    /* can't have trailing deliminator */
-    free(buf);
-    return "Improper format list.";
-  }
-#else
-  if(need_item){    /* allow 1 trailing deliminator */
-    *--walk='\0';  /* remove the trailing deliminator */
-  }
-#endif
-  /*** actually parse the list ***/
-  walk = buf;
-  while(items--){
-    format_node *endp;
-    char *equal_loc;
-    sep_loc = strpbrk(walk," ,\t\n");
-    /* if items left, then sep_loc is not in header override */
-    if(items && sep_loc) *sep_loc = '\0';
-    equal_loc = strpbrk(walk,"=");
-    if(equal_loc){   /* if header override */
-      *equal_loc = '\0';
-      equal_loc++;
-    }
-    fnode =  do_one_spec(walk,equal_loc);
-    if(!fnode){
-      free(buf);
-      return "Unknown user-defined format specifier.";
-    }
-    endp = fnode; while(endp->next) endp = endp->next;  /* find end */
-    endp->next = sfn->f_cooked;
-    sfn->f_cooked = fnode;
-    walk = sep_loc + 1; /* point to next item, if any */
-  }
-  free(buf);
-  already_parsed_format = 1;
-  return NULL;
-}
-
-
-
-
-/*
+/******************************************************************
  * Used to parse option AIX field descriptors.
  * Put each completed format_node onto the list starting at ->f_cooked
  */
@@ -204,8 +115,6 @@ static const char *aix_format_parse(sf_node *sfn){
   char *walk;
   int items;
 
-trace("aix_format_parse called\n");
-
   /*** sanity check and count items ***/
   items = 0;
   walk = sfn->sf;
@@ -285,6 +194,106 @@ double_percent:
   return NULL;
 }
 
+/***************************************************************
+ * Used to parse option O lists. Option O is shared between
+ * sorting and formatting. Users may expect one or the other.
+ * The "broken" flag enables a really bad Unix98 misfeature.
+ * Put each completed format_node onto the list starting at ->f_cooked
+ */
+static const char *format_parse(sf_node *sfn){
+  char *buf;                   /* temp copy of arg to hack on */
+  char *sep_loc;               /* separator location: " \t,\n" */
+  char *walk;
+  const char *err;       /* error code that could or did happen */
+  format_node *fnode;
+  int items;
+  int need_item;
+  static char errbuf[80]; /* for variable-text error message */
+
+  /*** prepare to operate ***/
+  buf = malloc(strlen(sfn->sf)+1);
+  strcpy(buf, sfn->sf);
+  
+  /*** sanity check and count items ***/
+  need_item = 1; /* true */
+  items = 0;
+  walk = buf;
+  do{
+    switch(*walk){
+    case ' ': case ',': case '\t': case '\n': case '\0':
+    /* Linux extension: allow \t and \n as delimiters */
+      if(need_item){
+        free(buf);
+        goto improper;
+      }
+      need_item=1;
+      break;
+    case '=':
+      if(broken) goto out;
+      /* fall through */
+    default:
+      if(need_item) items++;
+      need_item=0;
+    }
+  } while (*++walk);
+out:
+  if(!items){
+    free(buf);
+    goto empty;
+  }
+#ifdef STRICT_LIST
+  if(need_item){    /* can't have trailing deliminator */
+    free(buf);
+    goto improper;
+  }
+#else
+  if(need_item){    /* allow 1 trailing deliminator */
+    *--walk='\0';  /* remove the trailing deliminator */
+  }
+#endif
+  /*** actually parse the list ***/
+  walk = buf;
+  while(items--){
+    format_node *endp;
+    char *equal_loc;
+    sep_loc = strpbrk(walk," ,\t\n");
+    /* if items left, then sep_loc is not in header override */
+    if(items && sep_loc) *sep_loc = '\0';
+    equal_loc = strpbrk(walk,"=");
+    if(equal_loc){   /* if header override */
+      *equal_loc = '\0';
+      equal_loc++;
+    }
+    fnode =  do_one_spec(walk,equal_loc);
+    if(!fnode){
+      if(!*errbuf){  /* if didn't already create an error string */
+        snprintf(
+          errbuf,
+          sizeof(errbuf),
+          "Unknown user-defined format specifier \"%s\".",
+          walk
+        );
+      }
+      free(buf);
+      goto unknown;
+    }
+    endp = fnode; while(endp->next) endp = endp->next;  /* find end */
+    endp->next = sfn->f_cooked;
+    sfn->f_cooked = fnode;
+    walk = sep_loc + 1; /* point to next item, if any */
+  }
+  free(buf);
+  already_parsed_format = 1;
+  return NULL;
+
+  /* errors may cause a retry looking for AIX format codes */
+  if(0) unknown:  err=errbuf;
+  if(0) empty:    err="Empty format list.";
+  if(0) improper: err="Improper format list.";
+  if(strchr(sfn->sf,'%')) err = aix_format_parse(sfn);
+  return err;
+}
+
 /****************  Parse single sort specifier *******************/
 static sort_node *do_one_sort_spec(const char *spec){
   const format_struct *fs;
@@ -306,7 +315,7 @@ static sort_node *do_one_sort_spec(const char *spec){
 }
 
 
-/*
+/**************************************************************
  * Used to parse long sorting options.
  * Put each completed sort_node onto the list starting at ->s_cooked
  */
@@ -468,7 +477,6 @@ static const char *short_sort_parse(sf_node *sfn){
  */
 static const char *parse_O_option(sf_node *sfn){
   const char *err;     /* error code that could or did happen */
-  const char *err2;
 
   if(sfn->next){
     err = parse_O_option(sfn->next);
@@ -478,17 +486,12 @@ static const char *parse_O_option(sf_node *sfn){
   switch(sfn->sf_code){
     case SF_B_o: case SF_G_format: case SF_U_o: /*** format ***/
       err = format_parse(sfn);
-      trace("Foo!\n");
-      if(err && strchr(sfn->sf,'%')) err = aix_format_parse(sfn);
-      trace("Bar!\n");
       if(!err) already_parsed_format = 1;
-      trace("Baz!\n");
       break;
     case SF_U_O:                                /*** format ***/
       /* Can have -l -f f u... set already_parsed_format like DEC does */
       if(already_parsed_format) return "option -O can not follow other format options.";
       err = format_parse(sfn);
-      if(err && strchr(sfn->sf,'%')) err = aix_format_parse(sfn);
       if(err) return err;
       already_parsed_format = 1;
       O_wrap(sfn,'u'); /* must wrap user format in default */
@@ -505,14 +508,7 @@ static const char *parse_O_option(sf_node *sfn){
         err = "option O is neither first format nor sort order.";
         break;
       }
-      err2 = format_parse(sfn);
-      if(!err2){ /* success as format code */
-        already_parsed_format = 1;
-        O_wrap(sfn,'b'); /* must wrap user format in default */
-        return NULL;
-      }
-      if(strchr(sfn->sf,'%')) err2 = aix_format_parse(sfn);
-      if(!err2){ /* success as format code */
+      if(!format_parse(sfn)){ /* if success as format code */
         already_parsed_format = 1;
         O_wrap(sfn,'b'); /* must wrap user format in default */
         return NULL;
@@ -577,7 +573,7 @@ int defer_sf_option(const char *arg, int source){
   return 0;                         /* boring, Unix98 is no change */
 }
 
-/* Since ps is not long-lived, the memory leak can be ignored. */
+/***** Since ps is not long-lived, the memory leak can be ignored. ******/
 void reset_sortformat(void){
   sf_list = NULL;          /* deferred sorting and formatting */
   format_list = NULL;      /* digested formatting options */
@@ -588,7 +584,7 @@ void reset_sortformat(void){
 }
 
 
-/* Search format_list for findme, then insert putme after findme. */
+/***** Search format_list for findme, then insert putme after findme. ****/
 static int fmt_add_after(const char *findme, format_node *putme){
   format_node *walk;
   if(!strcmp(format_list->name, findme)){
@@ -608,7 +604,7 @@ static int fmt_add_after(const char *findme, format_node *putme){
   return 0; /* fail */
 }
 
-/* Search format_list for findme, then delete it. */
+/******* Search format_list for findme, then delete it. ********/
 static int fmt_delete(const char *findme){
   format_node *walk;
   format_node *old;
@@ -650,8 +646,18 @@ static const char *generate_sysv_list(void){
   }
   PUSH("tname");  /* Unix98 says "TTY" here, yet "tty" produces "TT". */
   if(format_flags & FF_Uf) PUSH("stime");
+  /* avoid duplicate columns from -FP and -Fly */
+  if(format_modifiers & FM_F){
+    /* if -FP take the Sun-style column instead (sorry about "sgi_p") */
+    if(!(format_modifiers & FM_P)) PUSH("psr");  /* should be ENG */
+    /* if -Fly take the ADDR-replacement RSS instead */
+    if(!( (format_flags & FF_Ul) && (format_modifiers & FM_y) )) PUSH("rss");
+  }
   if(format_flags & FF_Ul){
     PUSH("wchan");
+  }
+  /* since FM_y adds RSS anyway, don't do this hack when that is true */
+  if( (format_flags & FF_Ul) && !(format_modifiers & FM_y) ){
     if(personality & PER_IRIX_l){ /* add "rss" then ':' here */
       PUSH("sgi_rss");
       fn = malloc(sizeof(format_node));
@@ -665,7 +671,11 @@ static const char *generate_sysv_list(void){
       fn->next = format_list;
       format_list=fn;
     }
+  }
+  if((format_modifiers & FM_F) || (format_flags & FF_Ul)){
     PUSH("sz");
+  }
+  if(format_flags & FF_Ul){
     if(format_modifiers & FM_y) PUSH("rss");
     else if(personality & (PER_ZAP_ADDR|PER_IRIX_l)) PUSH("sgi_p");
     else PUSH("addr_1");
@@ -684,6 +694,7 @@ static const char *generate_sysv_list(void){
     PUSH("pgid");
   }
   if(format_flags & (FF_Uf|FF_Ul)) PUSH("ppid");
+  if(format_modifiers & FM_T) PUSH("spid");
   PUSH("pid");
   if(format_flags & FF_Uf){
     if(personality & PER_SANE_USER) PUSH("user");
