@@ -67,25 +67,6 @@ const char *ERR_PRELOAD_FILE = "error: unable to open preload file '%s'\n";
 const char *WARN_BAD_LINE = "warning: %s(%d): invalid syntax, continuing...\n";
 
 
-#define DOTSLASH(x) do{ \
-  char *p_ = (x);           \
-  for(;;){                  \
-    p_ = strchr(p_, '.');   \
-    if(!p_) break;          \
-    *p_ = '/';              \
-  }                         \
-}while(0)
-
-#define SLASHDOT(x) do{ \
-  char *p_ = (x);           \
-  for(;;){                  \
-    p_ = strchr(p_, '.');   \
-    if(!p_) break;          \
-    *p_ = '/';              \
-  }                         \
-}while(0)
-
-
 /*
  *    Main... 
  *
@@ -94,6 +75,7 @@ int main(int argc, char **argv) {
 const char *me = (const char *)basename(argv[0]);
 bool SwitchesAllowed = true;
 bool WriteMode = false;
+int ReturnCode = 0;
 const char *preloadfile = DEFAULT_PRELOAD;
 
    PrintName = true;
@@ -137,12 +119,12 @@ const char *preloadfile = DEFAULT_PRELOAD;
       } else {
          SwitchesAllowed = false;
          if (WriteMode)
-            WriteSetting(*argv);
+            ReturnCode = WriteSetting(*argv);
          else ReadSetting(*argv);
       } /* end if */
    } /* end for */      
 
-return 0;
+return ReturnCode;
 } /* end main */
 
 
@@ -207,8 +189,7 @@ char *name, *value;
       return;
    } /* endif */
 
-   while (!feof(fp)) {
-      fgets(oneline, 256, fp);
+   while (fgets(oneline, 256, fp)) {
       oneline[256] = 0;
       n++;
       t = StripLeadingAndTrailingSpaces(oneline);
@@ -255,6 +236,7 @@ const char *name = setting;
 const char *value;
 const char *equals;
 char *tmpname;
+char *cptr;
 FILE *fp;
 char *outname;
 
@@ -285,8 +267,13 @@ char *outname;
    strncpy(outname, name, (int)(equals-name)); 
    outname[equals-name] = 0;
  
-   DOTSLASH(tmpname); /* change . to / */
-   SLASHDOT(outname); /* change / to . */
+   while (cptr = strchr(tmpname, '.')) {      /* change . to / */
+      *cptr = '/';
+   } /* endwhile */
+
+   while (cptr = strchr(outname, '/')) {      /* change / to . */
+      *cptr = '.';
+   } /* endwhile */
 
    fp = fopen(tmpname, "w");
 
@@ -324,7 +311,7 @@ return rc;
  */
 int ReadSetting(const char *setting) {
 int rc = 0;
-char *tmpname, *outname;
+char *tmpname, *outname, *cptr;
 char inbuf[1025];
 const char *name = setting;
 FILE *fp;
@@ -340,9 +327,13 @@ FILE *fp;
    strcat(tmpname, name); 
    strcpy(outname, name); 
  
+   while (cptr = strchr(tmpname, '.')) {      /* change . to / */
+      *cptr = '/';
+   } /* endwhile */
 
-   DOTSLASH(tmpname); /* change . to / */
-   SLASHDOT(outname); /* change / to . */
+   while (cptr = strchr(outname, '/')) {      /* change / to . */
+      *cptr = '.';
+   } /* endwhile */
 
    fp = fopen(tmpname, "r");
 
@@ -394,7 +385,7 @@ struct stat ts;
       rc = -1;
    } else {
       readdir(dp); readdir(dp);   /* skip . and .. */
-      while (( de = readdir(dp) )) {
+      while (de = readdir(dp)) {
          tmpdir = (char *)malloc(strlen(path)+strlen(de->d_name)+2);
          sprintf(tmpdir, "%s%s", path, de->d_name);
          rc2 = stat(tmpdir, &ts);       /* should check this return code */
@@ -415,4 +406,3 @@ struct stat ts;
 
 return rc;
 } /* end DisplayAll() */
-
