@@ -72,6 +72,13 @@
  * Modified 1999/10/22 Tim Janik <timj@gtk.org>
  * miscellaneous minor fixes, including "usage: ..." output for
  * unrecognized options.
+ *
+ * Modified 2000/02/07 Jakub Jelinek <jakub@redhat.com>
+ * Only load System.map when we are going to display WCHAN.
+ * Show possible error messages from that load using SHOWMESSAGE.
+ *
+ * Modified 2000/07/10 Michael K. Johnson <johnsonm@redhat.com>
+ * Integrated a patch to display SMP information.
  */
 
 #include <errno.h>
@@ -81,6 +88,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <fcntl.h>
+#include <libintl.h>
 #include <time.h>
 #include <sys/ioctl.h>
 #include <pwd.h>
@@ -97,18 +105,25 @@
 #include "proc/sysinfo.h"
 #include "proc/procps.h"
 #include "proc/whattime.h"
-#include "proc/sig.h"
+#include "proc/signals.h"
 #include "proc/version.h"
 #include "proc/readproc.h"
 #include "proc/status.h"
 #include "proc/devname.h"
-#include "proc/compare.h"
+/* these should be in the readproc.h header or in the procps.h header */
+typedef int (*cmp_t)(void*,void*);
+extern void reset_sort_options (void);
+extern int parse_sort_opt(char* opt);
+extern void register_sort_function (int dir, cmp_t func);
 
 #define PUTP(x) (tputs(x,1,putchar))
 #define BAD_INPUT -30
 
 #include "top.h"  /* new header for top specific things */
 
+static int *cpu_mapping;
+static int nr_cpu;
+
 /*#######################################################################
  *####  Startup routines: parse_options, get_options,      ##############
  *####                    setup_terminal and main          ##############
@@ -120,7 +135,7 @@
        * if top is in secure mode, disallow changing of the delay time between
        * screen updates.
        */
-static void parse_options(char *Options, int secure)
+void parse_options(char *Options, int secure)
 {
     int i;
     for (i = 0; i < strlen(Options); i++) {
@@ -187,6 +202,9 @@ static void parse_options(char *Options, int secure)
 	    break;
 	  case '\n':
 	    break;
+	  case 'I':
+	    Irixmode = 0;
+	    break;
 	  default:
 	    fprintf(stderr, "Wrong configuration option %c\n", i);
 	    exit(1);
@@ -217,14 +235,19 @@ static void parse_options(char *Options, int secure)
  * the 'W' command. 
  */
 
-static void get_options(void)
+void get_options(void)
 {
     FILE *fp;
     char *pt;
     char rcfile[MAXNAMELEN];
     char Options[256] = "";
+    int i;
 
-    header_lines = 7;
+    nr_cpu = sysconf (_SC_NPROCESSORS_ONLN);
+    cpu_mapping = (int *) xmalloc (sizeof (int) * nr_cpu);
+    /* read cpuname */
+    for (i=0; i< nr_cpu; i++) cpu_mapping[i]=i;
+    header_lines = 6 + nr_cpu;
     strcpy(rcfile, SYS_TOPRC);
     fp = fopen(rcfile, "r");
     if (fp != NULL) {
@@ -253,9 +276,9 @@ static void get_options(void)
 }
 
 /*
- * Set up the terminal attributes.
- */
-static void setup_terminal(void)
+     * Set up the terminal attributes.
+     */
+void setup_terminal(void)
 {
     char *termtype;
     struct termios newtty;
@@ -308,44 +331,6 @@ static void setup_terminal(void)
     tcgetattr(0, &Rawtty);
 }
 
-static int parseint(const char *src, const char *err)
-{
-    char *endp;
-    int num;
-    int len;
-    num = strtol(src, &endp, 0);
-    if (*endp == '\0') return num;
-    /* also accept prefixes of: infinite, infinity, maximum, all */
-    len = strlen(src);
-    if(len<1) goto fail;
-    if(len<9 && !strncmp(src,"infinite",len)) return INT_MAX;
-    if(len<9 && !strncmp(src,"infinity",len)) return INT_MAX;
-    if(len<8 && !strncmp(src,"maximum" ,len)) return INT_MAX;
-    if(len<4 && !strncmp(src,"all"     ,len)) return INT_MAX;
-fail:
-    fprintf(stderr, err, src);
-    exit(1);
-}
-
-static double parseflt(const char *src, const char *err)
-{
-    char *endp;
-    double num;
-    int len;
-    num = strtod(src, &endp);
-    if (*endp == '\0') return num;
-    /* also accept prefixes of: infinite, infinity, maximum, all */
-    len = strlen(src);
-    if(len<1) goto fail;
-    if(len<9 && !strncmp(src,"infinite",len)) return (double)INT_MAX;
-    if(len<9 && !strncmp(src,"infinity",len)) return (double)INT_MAX;
-    if(len<8 && !strncmp(src,"maximum" ,len)) return (double)INT_MAX;
-    if(len<4 && !strncmp(src,"all"     ,len)) return (double)INT_MAX;
-fail:
-    fprintf(stderr, err, src);
-    exit(1);
-}
-
 int main(int argc, char **argv)
 {
     /* For select(2). */
@@ -358,6 +343,7 @@ int main(int argc, char **argv)
 
     struct sigaction sact;
 
+    setlocale(LC_ALL, "");
     get_options();
     
     /* set to PCPU sorting */
@@ -366,7 +352,6 @@ int main(int argc, char **argv)
     /*
      * Parse arguments.
      */
-    (void)argc;
     argv++;
     while (*argv) {
 	cp = *argv++;
@@ -374,10 +359,16 @@ int main(int argc, char **argv)
 	    switch (*cp) {
 	      case 'd':
 	        if (cp[1]) {
-	            Sleeptime = parseflt(++cp, PROGNAME ": Bad delay time %s'\n");
+		    if (sscanf(++cp, "%f", &Sleeptime) != 1) {
+			fprintf(stderr, PROGNAME ": Bad delay time `%s'\n", cp);
+			exit(1);
+		    }
 		    goto breakargv;
 		} else if (*argv) { /* last char in an argv, use next as arg */
-		    Sleeptime = parseflt(cp = *argv++, PROGNAME ": Bad delay time %s'\n");
+		    if (sscanf(cp = *argv++, "%f", &Sleeptime) != 1) {
+			fprintf(stderr, PROGNAME ": Bad delay time `%s'\n", cp);
+			exit(1);
+		    }
 		    goto breakargv;
 		} else {
 		    fprintf(stderr, "-d requires an argument\n");
@@ -386,11 +377,17 @@ int main(int argc, char **argv)
 		break;
 	      case 'n':
 		if (cp[1]) {
-		    Loops = parseint(++cp, PROGNAME ": Bad value %s'\n");
+	   	    if (sscanf(++cp, "%d", &Loops) != 1) {
+			fprintf(stderr, PROGNAME ": Bad value `%s'\n", cp);
+			exit(1);
+		    }
 		    goto breakargv;
 		} else if (*argv) { /* last char in an argv, use next as arg */
-		    Loops = parseint(cp = *argv++, PROGNAME ": Bad value %s'\n");
-		    goto breakargv;
+		    if (sscanf(cp = *argv++, "%d", &Loops) != 1) {
+			fprintf(stderr, PROGNAME ": Bad value `%s'\n", cp);
+			 exit(1);
+	 	     }
+		     goto breakargv;
 		}
 		break;
 					
@@ -450,26 +447,30 @@ int main(int argc, char **argv)
 	      case 's':
 		  Secure = 1;
 		  break;
+	      case 'C':
+		  CPU_states = 1;
+		  break;
 	      case '-':
 		break;		/* Just ignore it */
+ 	      case 'v':
+ 	      case 'V':
+ 		fprintf(stdout, "top (%s)\n", procps_version);
+ 		exit(0);
+	      case 'h':
+ 		fprintf(stdout, "usage: " PROGNAME " -hvbcisqS -d delay -p pid -n iterations\n");
+		exit(0);
 	      default:
 		fprintf(stderr, PROGNAME ": Unknown argument `%c'\n", *cp);
-		/* fall through */
-	      case 'h':
-		fprintf(stderr, "usage: " PROGNAME " -bcisqS -d delay -p pid -n iterations\n");
-		exit(1);
+ 		fprintf(stdout, "usage: " PROGNAME " -hvbcisqS -d delay -p pid -n iterations\n");
+ 	        exit(1);
 	    }
 	    cp++;
 	}
     breakargv:
     }
     
-    /* for correct handling of some fields, we have to do distinguish 
-     * between kernel versions */
-    /* get kernel symbol table, if needed */
-    if (!CL_wchan_nout) {
-	if (open_psdb(NULL)) CL_wchan_nout = 1;
-    }
+    if (nr_cpu > 1 && CPU_states)
+      header_lines++;
 
     setup_terminal();
     window_size(0);
@@ -522,10 +523,12 @@ int main(int argc, char **argv)
 	/*
 	 *  end when exiting with an error.
 	 */
-static void error_end(int rno)
+void error_end(int rno)
 {
+    if (psdbsucc)
+        close_psdb();
     if (!Batch)
-       tcsetattr(0, TCSAFLUSH, &Savetty);
+	tcsetattr(0, TCSAFLUSH, &Savetty);
     PUTP(tgoto(cm, 0, Lines - 1));
     fputs("\r\n", stdout);
     exit(rno);
@@ -533,11 +536,12 @@ static void error_end(int rno)
 /*
 	 * Normal end of execution.
 	 */
-static void end(int signo)
+void end(int signo)
 {
-    (void)signo;
+    if (psdbsucc)
+	close_psdb();
     if (!Batch)
-       tcsetattr(0, TCSAFLUSH, &Savetty);
+	tcsetattr(0, TCSAFLUSH, &Savetty);
     PUTP(tgoto(cm, 0, Lines - 1));
     fputs("\r\n", stdout);
     exit(0);
@@ -546,18 +550,17 @@ static void end(int signo)
 /*
 	 * SIGTSTP catcher.
 	 */
-static void stop(int signo)
+void stop(int signo)
 {
-    (void)signo;
     /* Reset terminal. */
     if (!Batch)
-       tcsetattr(0, TCSAFLUSH, &Savetty);
+	tcsetattr(0, TCSAFLUSH, &Savetty);
     PUTP(tgoto(cm, 0, Lines - 3));
     fflush(stdout);
     raise(SIGSTOP);
     /* Later... */
     if (!Batch)
-       tcsetattr (0, TCSAFLUSH, &Rawtty);
+	tcsetattr (0, TCSAFLUSH, &Rawtty);
 }
 
 /*
@@ -565,10 +568,10 @@ static void stop(int signo)
        * and also catches SIGWINCHs, and adjusts Maxlines.  Basically, this is
        * the central place for window size stuff.
        */
-static void window_size(int signo)
+void window_size(int signo)
 {
     struct winsize ws;
-    (void)signo;
+
     if((ioctl(1, TIOCGWINSZ, &ws) != -1) && (ws.ws_col>73) && (ws.ws_row>7)){
 	Cols = ws.ws_col;
 	Lines = ws.ws_row;
@@ -583,17 +586,44 @@ static void window_size(int signo)
      */
      Numfields = make_header();
 }
+
+/*
+	* this prints a possible message from open_psdb_message
+	*/
+static void top_message(const char *format, ...) {
+    va_list arg;
+    int n;
+    char buffer[512];
+
+    va_start (arg, format);
+    n = vsnprintf (buffer, 512, format, arg);
+    va_end (arg);
+    if (n > -1 && n < 512)
+	SHOWMESSAGE(("%s", buffer));
+}
+
 /*
        * this adjusts the lines needed for the header to the current value
        */
-static int make_header(void)
+int make_header(void)
 {
     int i, j;
 
     j = 0;
     for (i = 0; i < strlen(Fields); i++) {
-	if (isupper(Fields[i])) {
+	if (Fields[i] < 'a') {
 	    pflags[j++] = Fields[i] - 'A';
+	    if (Fields[i] == 'U' && CL_wchan_nout == -1) {
+		CL_wchan_nout = 0;
+		/* for correct handling of WCHAN fields, we have to do distinguish 
+		 * between kernel versions */
+		/* get kernel symbol table, if needed */
+		if (open_psdb_message(NULL, top_message)) {
+		    CL_wchan_nout = 1;
+		} else {
+		    psdbsucc = 1;
+		}
+	    }
 	}
     }
     strcpy(Header, "");
@@ -615,7 +645,7 @@ static int make_header(void)
  * returned is a statically allocated buffer, so don't expect it to
  * persist between calls.
  */
-static char *getstr(void)
+char *getstr(void)
 {
     static char line[BUFSIZ];	/* BUFSIZ from <stdio.h>; arbitrary */
     int i = 0;
@@ -634,10 +664,10 @@ static char *getstr(void)
 
 
 /*
- * Get an integer from the user.  Display an error message and
- * return BAD_INPUT if it's invalid; else return the number.
+ * Get an integer from the user.  Display an error message and return BAD_INPUT
+ * if it's invalid; else return the number.
  */
-static int getint(void)
+int getint(void)
 {
     char *line;
     int i;
@@ -662,9 +692,9 @@ static int getint(void)
 
 
 /*
- * Get a float from the user.  Just like getint().
- */
-static float getfloat(void)
+	 * Get a float from the user.  Just like getint().
+	 */
+float getfloat(void)
 {
     char *line;
     int i;
@@ -692,13 +722,13 @@ static float getfloat(void)
 	 * Get a signal number or name from the user.  Return the number, or -1
 	 * on error.
 	 */
-static int getsig(void)
+int getsig(void)
 {
     char *line;
 
     /* This is easy. */
     line = getstr();
-    return signal_name_to_number(line);
+    return (get_signal2(line));
 }
 
 /*#######################################################################
@@ -710,7 +740,7 @@ static int getsig(void)
  *#######################################################################
  */
 
-static int time_sort (proc_t **P, proc_t **Q)
+int time_sort (proc_t **P, proc_t **Q)
 {
     if (Cumulative) {
 	if( ((*P)->cutime + (*P)->cstime + (*P)->utime + (*P)->stime) < 
@@ -728,14 +758,14 @@ static int time_sort (proc_t **P, proc_t **Q)
     return 0;
 }
 
-static int pcpu_sort (proc_t **P, proc_t **Q)
+int pcpu_sort (proc_t **P, proc_t **Q)
 {
     if( (*P)->pcpu < (*Q)->pcpu )      return -1;
     if( (*P)->pcpu > (*Q)->pcpu )      return 1;
     return 0;
 }
 
-static int mem_sort (proc_t **P, proc_t **Q)
+int mem_sort (proc_t **P, proc_t **Q)
 {
     if( (*P)->resident < (*Q)->resident )      return -1;
     if( (*P)->resident > (*Q)->resident )      return 1;  
@@ -762,7 +792,7 @@ int age_sort (proc_t **P, proc_t **Q)
 	 * The description of a displayed field is marked by a leading 
 	 * asterisk (*).
 	 */
-static void show_fields(void)
+void show_fields(void)
 {
     int i, row, col;
     char *p;
@@ -783,7 +813,7 @@ static void show_fields(void)
 /*
 	 * change order of displayed fields
 	 */
-static void change_order(void)
+void change_order(void)
 {
     char c, ch, *p;
     int i;
@@ -831,7 +861,7 @@ static void change_order(void)
 /*
 	 * toggle displayed fields
 	 */
-static void change_fields(void)
+void change_fields(void)
 {
     int i, changed = 0;
     int row, col;
@@ -848,13 +878,13 @@ static void change_fields(void)
 	PUTP(me);
 	putchar('\n');
 	PUTP(tgoto(cm, 0, 1));
+	printf("Toggle fields with a-x, any other key to return: ");
+	fflush(stdout);
 	if (!Batch) { /* should always be true, but... */
-	    printf("Toggle fields with a-x, any other key to return: ");
-	    fflush(stdout);
 	    tcsetattr(0, TCSAFLUSH, &Rawtty);
+	    read(0, &c, 1);
+	    tcsetattr(0, TCSAFLUSH, &Savetty);
 	}
-	read(0, &c, 1);
-	tcsetattr(0, TCSAFLUSH, &Savetty);
 	i = toupper(c) - 'A';
 	if (i >= 0 && i < sizeof headers / sizeof headers[0]) {
 	    row = i % (Lines - 3) + 3;
@@ -880,40 +910,45 @@ static void change_fields(void)
 	Numfields = make_header();
 }
 
-/* Do the scaling stuff: interprets time in seconds, formats it to
- * fit width, and returns pointer to static char*.
- */
-static char *scale_time(int t,int width) 
+/* Do the scaling stuff
+   scale_time(time,width) - interprets time in seconds, formats it to fit width
+
+   Both return pointer to static char*.
+*/
+
+char *scale_time(int time,int width) 
 {
 	static char buf[100];
 
 	/* Try successively higher units until it fits */
 
-	sprintf(buf,"%d:%02d",t/60,t%60);	/* minutes:seconds */
+	sprintf(buf,"%d:%02d",time/60,time%60);	/* minutes:seconds */
 	if (strlen(buf)<=width) 
 		return buf;
 
-	t/=60;	/* minutes */
-	sprintf(buf,"%dm",t);
+	time/=60;	/* minutes */
+	sprintf(buf,"%dm",time);
 	if (strlen(buf)<=width) 
 		return buf;
 
-	t/=60;	/* hours */
-	sprintf(buf,"%dh",t);
+	time/=60;	/* hours */
+	sprintf(buf,"%dh",time);
 	if (strlen(buf)<=width) 
 		return buf;
 
-	t/=24;	/* days */
-	sprintf(buf,"%dd",t);
+	time/=24;	/* days */
+	sprintf(buf,"%dd",time);
 	if (strlen(buf)<=width) 
 		return buf;
 	
-	t/=7;	/* weeks */
-	sprintf(buf,"%dw",t);
+	time/=7;	/* weeks */
+	sprintf(buf,"%dw",time);
 	return buf;	/* this is our last try; 
 				if it still doesn't fit, too bad. */
 
-	/* FIXME: if someone has a 16-way SMP running over a year... */
+	/* :-) I suppose if someone has a SMP version of Linux with a few
+           thousand processors, they could accumulate 18 years of CPU time... */
+           
 }
 
 /*   scale_k(k,width,unit)  - interprets k as a count, formats to fit width.
@@ -921,7 +956,7 @@ static char *scale_time(int t,int width)
 			    count; 2 for megabytes; 3 for gigabytes.
 */
 
-static char *scale_k(int k,int width,int unit) 
+char *scale_k(int k,int width,int unit) 
 {
 		/* kilobytes, megabytes, gigabytes, too-big-for-int-bytes */
 	static double scale[]={1024,1024*1024,1024*1024*1024,0};
@@ -959,7 +994,7 @@ static char *scale_k(int k,int width,int unit)
 	/*
 	 * Displays infos for a single task
 	 */
-static void show_task_info(proc_t *task, int pmem)
+void show_task_info(proc_t *task, int pmem)
 {
     int i,j;
     unsigned int t;
@@ -984,6 +1019,9 @@ static void show_task_info(proc_t *task, int pmem)
 	  case P_PCPU:
 	    sprintf(tmp, "%4.1f ", (float)task->pcpu / 10);
 	    break;
+	  case P_LCPU:
+	    sprintf(tmp, "%2d ", task->lproc);
+	    break;
 	  case P_PMEM:
 	    sprintf(tmp, "%4.1f ", (float)pmem / 10);
 	    break;
@@ -1071,9 +1109,6 @@ static void show_task_info(proc_t *task, int pmem)
 	    sprintf(tmp, "%s", cmdptr);
 	    tmp3[0]=0;
 	    break;
-	  case P_LTR:
-	    sprintf(tmp, "%4.4s ", scale_k((task->lrs << CL_pg_shift), 4, 1));
-	    break;
 	  case P_FLAGS:
 	    sprintf(tmp, "%8lx ", task->flags);
 	    break;
@@ -1103,13 +1138,14 @@ static void show_task_info(proc_t *task, int pmem)
  * would be useful to support the PROC_UID and PROC_TTY
  * as command line options.
  */
-static void show_procs(void)
+void show_procs(void)
 {
     static proc_t **p_table=NULL;
     static int proc_flags;
     int count;
     int ActualLines;
     float elapsed_time;
+    unsigned int main_mem;
     static int first=0;
 
     if (first==0) {
@@ -1139,7 +1175,7 @@ static void show_procs(void)
      * and sort the list. */
     do_stats(p_table, elapsed_time,1);
     /* Display the memory and swap space usage. */
-    show_meminfo();
+    main_mem = show_meminfo();
     if (strlen(Header) + 2 > Cols)
 	Header[Cols - 2] = 0;
     PUTP(mr);
@@ -1155,20 +1191,18 @@ static void show_procs(void)
     ActualLines = 0;
     while ((ActualLines < Maxlines) && (p_table[count]->pid!=-1)) {
 	int pmem;
-	char Stat;
+	char stat;
 
-	Stat = p_table[count]->state;
+	stat = p_table[count]->state;
 
-	if ( (!Noidle || (Stat != 'S' && Stat != 'Z')) &&
+	if ( (!Noidle || (stat != 'S' && stat != 'Z')) &&
 	     ( (CurrUser[0] == '\0') ||
-	      (!strcmp(CurrUser,p_table[count]->euser) ) ) ) {
+	      (!strcmp((char *)CurrUser,p_table[count]->euser) ) ) ) {
 
 	    /*
 	     * Show task info.
 	     */
-	    /* FIXME: this appears to assume a 4 kB page size */
-	    /* FIXME: maybe this can overflow on huge memory systems */
-	    pmem = p_table[count]->resident * 250 / (kb_main_total);
+	    pmem = p_table[count]->resident * 1000 / (main_mem / 4);
 	    show_task_info(p_table[count], pmem);
 	    if (!Batch)
 	    	ActualLines++;
@@ -1186,82 +1220,98 @@ static void show_procs(void)
  * elapsed since the last update. This is essential for computing
  * percent CPU usage.
  */
-static float get_elapsed_time(void)
+float get_elapsed_time(void)
 {
-    struct timeval t;
+    struct timeval time;
     static struct timeval oldtime;
     struct timezone timez;
     float elapsed_time;
 
-    gettimeofday(&t, &timez);
-    elapsed_time = (t.tv_sec - oldtime.tv_sec)
-	+ (float) (t.tv_usec - oldtime.tv_usec) / 1000000.0;
-    oldtime.tv_sec  = t.tv_sec;
-    oldtime.tv_usec = t.tv_usec;
+    gettimeofday(&time, &timez);
+    elapsed_time = (time.tv_sec - oldtime.tv_sec)
+	+ (float) (time.tv_usec - oldtime.tv_usec) / 1000000.0;
+    oldtime.tv_sec = time.tv_sec;
+    oldtime.tv_usec = time.tv_usec;
     return (elapsed_time);
 }
 
+
 /*
  * Reads the memory info and displays it.  Returns the total memory
  * available, for use in percent memory usage calculations.
  */
-static void show_meminfo(void)
+unsigned show_meminfo(void)
 {
-    meminfo();	/* read+parse /proc/meminfo */
+    unsigned long long **mem;
+
+    if (!(mem = meminfo()) ||	/* read+parse /proc/meminfo */
+	mem[meminfo_main][meminfo_total] == 0) {	/* cannot normalize mem usage */
+	fprintf(stderr, "Cannot get size of memory from /proc/meminfo\n");
+	error_end(1);
+    }
     if (show_memory) {
-	printf(
-	    "Mem:  %6dK av, %6dK used, %6dK free, %6dK shrd, %6dK buff",
-	    kb_main_total,
-	    kb_main_used,
-	    kb_main_free,
-	    kb_main_shared,
-	    kb_main_buffers
-	);
+	printf("Mem:  %7LdK av, %7LdK used, %7LdK free, %7LdK shrd, %7LdK buff",
+	       mem[meminfo_main][meminfo_total] >> 10,
+	       mem[meminfo_main][meminfo_used] >> 10,
+	       mem[meminfo_main][meminfo_free] >> 10,
+	       mem[meminfo_main][meminfo_shared] >> 10,
+	       mem[meminfo_main][meminfo_buffers] >> 10);
 	PUTP(top_clrtoeol);
 	putchar('\n');
-	printf(
-	    "Swap: %6dK av, %6dK used, %6dK free                %6dK cached",
-	    kb_swap_total,
-	    kb_swap_used,
-	    kb_swap_free,
-	    kb_main_cached
-	);
+	printf("Swap: %7LdK av, %7LdK used, %7LdK free                 %7LdK cached",
+	       mem[meminfo_swap][meminfo_total] >> 10,
+	       mem[meminfo_swap][meminfo_used] >> 10,
+	       mem[meminfo_swap][meminfo_free] >> 10,
+	       mem[meminfo_total][meminfo_cached] >> 10);
 	PUTP(top_clrtoeol);
 	putchar('\n');
     }
     PUTP(me);
     PUTP(top_clrtoeol);
     putchar('\n');
+    return mem[meminfo_main][meminfo_total] >> 10;
 }
 
-
 /*
  * Calculates the number of tasks in each state (running, sleeping, etc.).
  * Calculates the CPU time in each state (system, user, nice, etc).
  * Calculates percent cpu usage for each task.
  */
-static void do_stats(proc_t** p, float elapsed_time, int pass)
+void do_stats(proc_t** p, float elapsed_time, int pass)
 {
     proc_t *this;
-    int arrindex, total_time, i, n = 0;
+    int index, total_time, cpumap, i, n = 0;
     int sleeping = 0, stopped = 0, zombie = 0, running = 0;
-    unsigned long system_ticks = 0, user_ticks = 0, nice_ticks = 0, idle_ticks = 1000;
+    unsigned long system_ticks = 0, user_ticks = 0, nice_ticks = 0, idle_ticks;
     static int prev_count = 0;
-    int systime, usrtime;
- 
-     /* start with one 4K page as a reasonable allocate size */
-     static int save_history_size = sizeof(struct save_hist) * 204;
-     static struct save_hist *save_history;
-     struct save_hist *New_save_hist;
- 
-     if (!save_history)
-       save_history = xcalloc(NULL, save_history_size);
-     New_save_hist = xcalloc(NULL, save_history_size);
+    int stime, utime;
+
+    /* start with one 4K page as a reasonable allocate size */
+    static int save_history_size = sizeof(struct save_hist) * 204;
+    static struct save_hist *save_history;
+    struct save_hist *New_save_hist;
+    static int *s_ticks_o = NULL, *u_ticks_o = NULL,
+               *n_ticks_o = NULL, *i_ticks_o = NULL;
+    int s_ticks, u_ticks, n_ticks, i_ticks, t_ticks;
+    char str[128];
+    FILE *file;
+
+    if (!save_history)
+	save_history = xcalloc(NULL, save_history_size);
+    New_save_hist = xcalloc(NULL, save_history_size);
+
+    if(s_ticks_o == NULL) {
+      s_ticks_o = (int *)malloc(nr_cpu * sizeof(int));
+      u_ticks_o = (int *)malloc(nr_cpu * sizeof(int));
+      n_ticks_o = (int *)malloc(nr_cpu * sizeof(int));
+      i_ticks_o = (int *)malloc(nr_cpu * sizeof(int));
+    }
+    idle_ticks = 1000 * nr_cpu;
 
     /*
      * Make a pass through the data to get stats.
      */
-    arrindex = 0;
+    index = 0;
     while (p[n]->pid != -1) {
 	this = p[n];
 	switch (this->state) {
@@ -1285,27 +1335,27 @@ static void do_stats(proc_t** p, float elapsed_time, int pass)
 
 	/*
 	 * Calculate time in this process.  Time is sum of user time
-	 * (usrtime) plus system time (systime).
+	 * (utime) plus system time (stime).
 	 */
 	total_time = this->utime + this->stime;
-        if (arrindex > save_history_size) {
-            save_history_size *= 2;
-            save_history = xrealloc(save_history, save_history_size);
-            New_save_hist = xrealloc(New_save_hist, save_history_size);
-        }
-	New_save_hist[arrindex].ticks = total_time;
-	New_save_hist[arrindex].pid = this->pid;
-	systime = this->stime;
-	usrtime = this->utime;
-	New_save_hist[arrindex].stime = systime;
-	New_save_hist[arrindex].utime = usrtime;
+	if (index * sizeof(struct save_hist) >= save_history_size) {
+	    save_history_size *= 2;
+	    save_history = xrealloc(save_history, save_history_size);
+	    New_save_hist = xrealloc(New_save_hist, save_history_size);
+	}
+	New_save_hist[index].ticks = total_time;
+	New_save_hist[index].pid = this->pid;
+	stime = this->stime;
+	utime = this->utime;
+	New_save_hist[index].stime = stime;
+	New_save_hist[index].utime = utime;
 
 	/* find matching entry from previous pass */
 	for (i = 0; i < prev_count; i++) {
 	    if (save_history[i].pid == this->pid) {
 		total_time -= save_history[i].ticks;
-		systime -= save_history[i].stime;
-		usrtime -= save_history[i].utime;
+		stime -= save_history[i].stime;
+		utime -= save_history[i].utime;
 
 		i = prev_count;
 	    }
@@ -1318,27 +1368,31 @@ static void do_stats(proc_t** p, float elapsed_time, int pass)
 	if (this->pcpu > 999)
 	    this->pcpu = 999;
 
-#if 0
 	/*
 	 * Calculate time in idle, system, user and niced tasks.
 	 */
 	idle_ticks -= this->pcpu;
-	system_ticks += systime;
-	user_ticks += usrtime;
+	system_ticks += stime;
+	user_ticks += utime;
 	if (this->nice > 0)
 	    nice_ticks += this->pcpu;
-#endif
 
-	arrindex++;
+	/*
+	 * If in Sun-mode, adjust cpu percentage not only for
+	 * the cpu the process is running on, but for all cpu together.
+	 */
+	if(!Irixmode) {
+	  this->pcpu /= nr_cpu;
+	  }
+	
+	index++;
 	n++;
     }
 
-#if 0
     if (idle_ticks < 0)
 	idle_ticks = 0;
     system_ticks = (system_ticks * 10 * 100/Hertz) / elapsed_time;
     user_ticks = (user_ticks * 10 * 100/Hertz) / elapsed_time;
-#endif
 
     /*
      * Display stats.
@@ -1349,28 +1403,106 @@ static void do_stats(proc_t** p, float elapsed_time, int pass)
 	       n, sleeping, running, zombie, stopped);
 	PUTP(top_clrtoeol);
 	putchar('\n');
-	/* BEGIN EXPERIMENTAL CODE */
-	/* Throw out the calculation above... TODO: remove calculation. */
-	four_cpu_numbers(&user_ticks,&nice_ticks,&system_ticks,&idle_ticks);
-	do{
-	  unsigned long sum;
-	  sum = user_ticks+nice_ticks+system_ticks+idle_ticks;
-	  user_ticks   = (user_ticks   * 1000) / sum;
-	  system_ticks = (system_ticks * 1000) / sum;
-	  nice_ticks   = (nice_ticks   * 1000) / sum;
-	  idle_ticks   = (idle_ticks   * 1000) / sum;
-	}while(0);
-	/* END EXPERIMENTAL CODE */
-	printf("CPU states:"
-	    " %2ld.%ld%% user, %2ld.%ld%% system,"
-	    " %2ld.%ld%% nice, %2ld.%ld%% idle",
-            user_ticks   / 10UL,   user_ticks   % 10UL,
-            system_ticks / 10UL,   system_ticks % 10UL,
-            nice_ticks   / 10UL,   nice_ticks   % 10UL,
-            idle_ticks   / 10UL,   idle_ticks   % 10UL
-	);
-	PUTP(top_clrtoeol);
-	putchar('\n');
+	if (nr_cpu == 1 || CPU_states) {
+	  /* BEGIN EXPERIMENTAL CODE */
+	  /* Throw out the calculation above... TODO: remove calculation. */
+	  four_cpu_numbers(&user_ticks,&nice_ticks,&system_ticks,&idle_ticks);
+	  do{
+	    unsigned long sum;
+	    sum = user_ticks+nice_ticks+system_ticks+idle_ticks;
+	    user_ticks   = (user_ticks   * 1000) / sum;
+	    system_ticks = (system_ticks * 1000) / sum;
+	    nice_ticks   = (nice_ticks   * 1000) / sum;
+	    idle_ticks   = (idle_ticks   * 1000) / sum;
+	  }while(0);
+	  /* END EXPERIMENTAL CODE */
+	  if (Irixmode) {
+	    user_ticks *= nr_cpu;
+	    system_ticks *= nr_cpu;
+	    nice_ticks *= nr_cpu;
+	    idle_ticks *= nr_cpu;
+	  }
+	  printf("CPU states:"
+		 " %2ld.%ld%% user, %2ld.%ld%% system,"
+		 " %2ld.%ld%% nice, %2ld.%ld%% idle",
+		 user_ticks / 10UL, user_ticks % 10UL,
+		 system_ticks / 10UL, system_ticks % 10UL,
+		 nice_ticks / 10UL, nice_ticks % 10UL,
+		 idle_ticks / 10UL, idle_ticks % 10UL);
+	  PUTP(top_clrtoeol);
+	  putchar('\n');
+	}
+	/*
+         * Calculate stats for all cpus
+         */
+	if(nr_cpu > 1) {
+	  file = fopen("/proc/stat", "r");
+	  if(file == NULL) {
+	    fprintf(stderr, "fopen failed on /proc/stat\n");
+	  }
+	  else {
+	    /* Skip the total CPU states. */
+	    if(fgets(str, 128, file) == NULL) {
+	      fprintf(stderr, "fgets failed on /proc/stat\n");
+	    }
+	    else {
+	      for(i = 0; i < nr_cpu; i++) {
+		if(fscanf(file, "cpu%*d %d %d %d %d\n",
+			  &u_ticks, &n_ticks, &s_ticks, &i_ticks) != 4) {
+		  fprintf(stderr, "fscanf failed on /proc/stat for cpu %d\n", i);
+		  break;
+		}
+		else  {
+		  t_ticks = (u_ticks + s_ticks + i_ticks + n_ticks)
+		  	    - (u_ticks_o[i] + s_ticks_o[i]
+			       + i_ticks_o[i] + n_ticks_o[i]);
+		  if (Irixmode) cpumap=i;
+		  else cpumap=cpu_mapping[i];
+		  printf ("CPU%d states: %2d.%-d%% user, %2d.%-d%% system,"
+			  " %2d.%-d%% nice, %2d.%-d%% idle",
+			  cpumap,
+			  (u_ticks - u_ticks_o[i] + n_ticks - n_ticks_o[i]) * 100 / t_ticks,
+			  (u_ticks - u_ticks_o[i]) * 100 % t_ticks / 100,
+			  (s_ticks - s_ticks_o[i]) * 100 / t_ticks,
+			  (s_ticks - s_ticks_o[i]) * 100 % t_ticks / 100,
+			  (n_ticks - n_ticks_o[i]) * 100 / t_ticks,
+			  (n_ticks - n_ticks_o[i]) * 100 % t_ticks / 100,
+			  (i_ticks - i_ticks_o[i]) * 100 / t_ticks,
+			  (i_ticks - i_ticks_o[i]) * 100 % t_ticks / 100);
+		  s_ticks_o[i] = s_ticks;
+		  u_ticks_o[i] = u_ticks;
+		  n_ticks_o[i] = n_ticks;
+		  i_ticks_o[i] = i_ticks;
+		  PUTP (top_clrtoeol);
+		  putchar ('\n');
+		}
+	      }
+	    }
+	    fclose(file);
+	  }
+	}
+    } else {
+      /*
+       * During the first pass, get the information for the
+       * different CPUs.
+       */
+      if(nr_cpu > 1) {
+       file = fopen("/proc/stat", "r");
+       if(file == NULL) {
+         puts("open");
+       }
+       if(fgets(str, 128, file) == NULL) {
+         fprintf(stderr, "fgets failed on /proc/stat\n");
+       }
+       for(i = 0; i < nr_cpu; i++) {
+         if(fscanf(file, "cpu%*d %d %d %d %d\n",
+                   &u_ticks_o[i], &n_ticks_o[i], &s_ticks_o[i],
+                   &i_ticks_o[i]) != 4) {
+           fprintf(stderr, "fscanf failed on /proc/stat for cpu %d\n", i);
+         }
+       }
+       fclose(file);
+      }
     }
     /*
      * Save this frame's information.
@@ -1392,7 +1524,7 @@ static void do_stats(proc_t** p, float elapsed_time, int pass)
 /*
  * Process keyboard input during the main loop
  */
-static void do_key(char c)
+void do_key(char c)
 {
     int numinput, i;
     char rcfile[MAXNAMELEN];
@@ -1408,7 +1540,11 @@ static void do_key(char c)
     else if (c == 12) {
 	clear_screen();
 	return;
+    } else if (c == 'I') {
+	Irixmode=(Irixmode) ? 0 : 1;
+	return;
     }
+
     /*
      * Switch the terminal to normal mode.  (Will the original
      * attributes always be normal?  Does it matter?  I suppose the
@@ -1454,22 +1590,21 @@ static void do_key(char c)
 	if (Secure)
 	    SHOWMESSAGE(("\aCan't kill in secure mode"));
 	else {
-	    int pid, signo;
+	    int pid, signal;
 	    PUTP(md);
 	    SHOWMESSAGE(("PID to kill: "));
 	    pid = getint();
 	    if (pid == BAD_INPUT)
 		break;
 	    PUTP(top_clrtoeol);
-	    SHOWMESSAGE(("Kill process %d with what signal? [15] ", pid));
+	    SHOWMESSAGE(("Kill PID %d with signal [15]: ", pid));
 	    PUTP(me);
-	    signo = getsig();
-	    /* FIXME: -1 may mean an unknown signal */
-	    if (signo == -1)
-		signo = SIGTERM;
-	    if (kill(pid, signo))
+	    signal = getsig();
+	    if (signal == -1)
+		signal = SIGTERM;
+	    if (kill(pid, signal))
 		SHOWMESSAGE(("\aKill of PID %d with %d failed: %s",
-			     pid, signo, strerror(errno)));
+			     pid, signal, strerror(errno)));
 	}
 	break;
       case 'l':
@@ -1504,7 +1639,7 @@ static void do_key(char c)
       case '#':
 	printf("Processes to display (0 for unlimited): ");
 	numinput = getint();
-	if (numinput != -1) {
+	if (numinput != BAD_INPUT) {
 	    Display_procs = numinput;
 	    window_size(0);
 	}
@@ -1583,7 +1718,7 @@ static void do_key(char c)
 	Numfields = make_header();
 	break;
       case 'T':
-	SHOWMESSAGE(("Sort by %s time", Cumulative ? "cumulative" : ""));
+	SHOWMESSAGE(("Sort by %stime", Cumulative ? "cumulative " : ""));
 	sort_type = S_TIME;
 	reset_sort_options();
 	register_sort_function( -1, (cmp_t)time_sort);	
@@ -1624,6 +1759,8 @@ static void do_key(char c)
 		    fprintf(fp, "%c", 'l');
 		if (!show_stats)
 		    fprintf(fp, "%c", 't');
+		if (!Irixmode)
+		    fprintf(fp, "%c", 'I');
 		fprintf(fp, "\n");
 		fclose(fp);
 		SHOWMESSAGE(("Wrote configuration to %s", rcfile));
@@ -1653,7 +1790,7 @@ static void do_key(char c)
  */
 #define Do(x) (flags & PROC_ ## x)
 
-static proc_t** readproctab2(int flags, proc_t** tab, ...) {
+proc_t** readproctab2(int flags, proc_t** tab, ...) {
     PROCTAB* PT = NULL;
     static proc_t *buff;
     int n = 0;
