@@ -54,7 +54,9 @@ int Numfields;
  * specified
  */
 #ifndef DEFAULT_SHOW
-#define DEFAULT_SHOW    "AbcDgHIjklMnoTPqrsuzVYEFWX"
+/*                       0         1         2         3 */
+/*                       0123456789012345678901234567890 */
+#define DEFAULT_SHOW    "AbcDgHIjklMnoTP|qrsuzyV{EFWX"
 #endif
 char Fields[256] = "";
 
@@ -93,12 +95,14 @@ float Sleeptime = 5;
 	/* for opening/closing the system map */
 int psdbsucc = 0;
 	/* Mode flags. */
+int Irixmode = 1;
 int Secure = 0;
 int Cumulative = 0;
 int Noidle = 0;
+int CPU_states = 0;
 char CurrUser[BUFSIZ];
 int CL_pg_shift = (PAGE_SHIFT - 10);
-int CL_wchan_nout = 0;
+int CL_wchan_nout = -1;
 int show_stats = 1;    /* show status summary */
 int show_memory = 1;   /* show memory summary */
 int show_loadav = 1;   /* show load average and uptime */
@@ -124,7 +128,7 @@ enum {
     P_SIZE, P_TRS, P_SWAP, P_SHARE,
     P_A, P_WP, P_DT, P_RSS,
     P_WCHAN, P_STAT, P_TIME, P_COMMAND,
-    P_LTR, P_FLAGS, P_END
+    P_LCPU, P_FLAGS, P_END
 };
 /* corresponding headers */
 char *headers[] =
@@ -137,7 +141,8 @@ char *headers[] =
     "SHARE ", "  A ", " WP ",
     "  D ", " RSS ", "WCHAN     ",
     "STAT ", "  TIME ", "COMMAND",
-    " LIB ", "   FLAGS "
+    "LC ",
+    "   FLAGS "
 };
 /* corresponding field desciptions */
 char *headers2[] =
@@ -150,7 +155,7 @@ char *headers2[] =
     "Shared Pages (kb)", "Accessed Page count", "Write Protected Pages",
     "Dirty Pages", "Resident Set Size (kb)", "Sleeping in Function",
     "Process Status", "CPU Time", "Command",
-    "Shared Library Pages used (kb) - does not work for ELF!!",
+    "Last used CPU (expect this to change regularly)",
     "Task Flags (see linux/sched.h)"
 };
 
@@ -168,6 +173,7 @@ oO\tChange order of displayed fields\n\
 h or ?\tPrint this list\n\
 S\tToggle cumulative mode\n\
 i\tToggle display of idle proceses\n\
+I\tToggle between Irix and Solaris views (SMP-only)\n\
 c\tToggle display of command name/line\n\
 l\tToggle display of load average\n\
 m\tToggle display of memory information\n\
