@@ -19,7 +19,7 @@
 */
 /* PROCPS
    This is part of the procps package maintained by Michael K. Johnson
-   <johnsonm@redhat.com>; report bugs to <procps-bugs@redhat.com>.
+   <johnsonm@redhat.com>; report bugs to <acahalan@cs.uml.edu>.
 */
    
 #include "proc/sysinfo.h"
@@ -39,12 +39,14 @@
 
 #define NDEBUG !DEBUG
 
-#define BUFFSIZE 1024
+#define BUFFSIZE 4096
 #define FALSE 0
 #define TRUE 1
 
 static char buff[BUFFSIZE]; /* used in the procedures */
 
+typedef unsigned long jiff;
+
 
 /****************************************************************/
 
@@ -82,7 +84,7 @@ static void showheader(void){
 	 "in","cs","us","sy","id");
 }
 
-static void getstat(unsigned *cuse, unsigned *cice, unsigned *csys, unsigned long *cide,
+static void getstat(jiff *cuse, jiff *cice, jiff *csys, jiff long *cide,
 	     unsigned *pin, unsigned *pout, unsigned *s_in, unsigned *sout,
 	     unsigned *itot, unsigned *i1, unsigned *ct) {
   static int Stat;
@@ -95,7 +97,7 @@ static void getstat(unsigned *cuse, unsigned *cice, unsigned *csys, unsigned lon
     *itot = 0; 
     *i1 = 1;   /* ensure assert below will fail if the sscanf bombs */
     b = strstr(buff, "cpu ");
-    sscanf(b, "cpu  %u %u %u %lu", cuse, cice, csys, cide);
+    sscanf(b, "cpu  %lu %lu %lu %lu", cuse, cice, csys, cide);
     b = strstr(buff, "page ");
     sscanf(b, "page %u %u", pin, pout);
     b = strstr(buff, "swap ");
@@ -167,13 +169,13 @@ int main(int argc, char *argv[]) {
   unsigned int tog=0; /* toggle switch for cleaner code */
   unsigned int i,hz;
   unsigned int running,blocked,swapped;
-  unsigned int cpu_use[2], cpu_nic[2], cpu_sys[2];
-  unsigned int duse,dsys,didl,Div,divo2;
-  unsigned long cpu_idl[2];
+  jiff cpu_use[2], cpu_nic[2], cpu_sys[2], cpu_idl[2];
+  jiff duse,dsys,didl,Div,divo2;
   unsigned int pgpgin[2], pgpgout[2], pswpin[2], pswpout[2];
   unsigned int inter[2],ticks[2],ctxt[2];
   unsigned int per=0, pero2; 
   unsigned long num=0;
+  unsigned int kb_per_page = sysconf(_SC_PAGESIZE) / 1024;
 
   setlinebuf(stdout);
   argc=0; /* redefined as number of integer arguments */
@@ -222,22 +224,25 @@ int main(int argc, char *argv[]) {
   getstat(cpu_use,cpu_nic,cpu_sys,cpu_idl,
 	  pgpgin,pgpgout,pswpin,pswpout,
 	  inter,ticks,ctxt);
-  duse= *(cpu_use)+ *(cpu_nic); 
-  dsys= *(cpu_sys);
-  didl= (*(cpu_idl))%UINT_MAX;
-  Div= (duse+dsys+didl);
+  duse= *cpu_use + *cpu_nic; 
+  dsys= *cpu_sys;
+  didl= *cpu_idl;
+  Div= duse+dsys+didl;
   hz=sysconf(_SC_CLK_TCK); /* get ticks/s from system */
-  divo2= Div/2;
+  divo2= Div/2UL;
   printf(format,
 	 running,blocked,swapped,
 	 kb_swap_used,kb_main_free,kb_main_buffers,kb_main_cached,
-	 (*(pswpin)*4*hz+divo2)/Div,
-	 (*(pswpout)*4*hz+divo2)/Div,
-	 (*(pgpgin)*hz+divo2)/Div,
-	 (*(pgpgout)*hz+divo2)/Div,
-	 (*(inter)*hz+divo2)/Div,
-	 (*(ctxt)*hz+divo2)/Div,
-	 (100*duse+divo2)/Div,(100*dsys+divo2)/Div,(100*didl+divo2)/Div);
+	 (*pswpin *kb_per_page*hz+divo2)/Div,
+	 (*pswpout*kb_per_page*hz+divo2)/Div,
+	 (*pgpgin             *hz+divo2)/Div,
+	 (*pgpgout            *hz+divo2)/Div,
+	 (*inter              *hz+divo2)/Div,
+	 (*ctxt               *hz+divo2)/Div,
+	 (100*duse+divo2)/Div,
+	 (100*dsys+divo2)/Div,
+	 (100*didl+divo2)/Div
+  );
 
   for(i=1;i<num;i++) { /* \\\\\\\\\\\\\\\\\\\\ main loop ////////////////// */
     sleep(per);
@@ -248,21 +253,26 @@ int main(int argc, char *argv[]) {
     getstat(cpu_use+tog,cpu_nic+tog,cpu_sys+tog,cpu_idl+tog,
 	  pgpgin+tog,pgpgout+tog,pswpin+tog,pswpout+tog,
 	  inter+tog,ticks+tog,ctxt+tog);
-    duse= *(cpu_use+tog)-*(cpu_use+!tog)+*(cpu_nic+tog)-*(cpu_nic+!tog);
-    dsys= *(cpu_sys+tog)-*(cpu_sys+!tog);
-    didl= (*(cpu_idl+tog)-*(cpu_idl+!tog))%UINT_MAX;
-    Div= (duse+dsys+didl);
-    divo2= Div/2;
+    duse= cpu_use[tog]-cpu_use[!tog] + cpu_nic[tog]-cpu_nic[!tog];
+    dsys= cpu_sys[tog]-cpu_sys[!tog];
+    didl= cpu_idl[tog]-cpu_idl[!tog];
+    /* idle can run backwards for a moment -- kernel "feature" */
+    if(cpu_idl[tog]<cpu_idl[!tog]) didl=0;
+    Div= duse+dsys+didl;
+    divo2= Div/2UL;
     printf(format,
 	   running,blocked,swapped,
 	   kb_swap_used,kb_main_free,kb_main_buffers,kb_main_cached,
-	   (((*(pswpin+tog)-*(pswpin+(!tog)))*4+pero2)/per),
-	   ((*(pswpout+tog)-*(pswpout+(!tog)))*4+pero2)/per,
-	   (*(pgpgin+tog)-*(pgpgin+(!tog))+pero2)/per,
-	   (*(pgpgout+tog)-*(pgpgout+(!tog))+pero2)/per,
-	   (*(inter+tog)-*(inter+(!tog))+pero2)/per,
-	   (*(ctxt+tog)-*(ctxt+(!tog))+pero2)/per,
-	   (100*duse+divo2)/Div,(100*dsys+divo2)/Div,(100*didl+divo2)/Div);
+	   ( (pswpin [tog]-pswpin [!tog])*kb_per_page+pero2 )/per,
+	   ( (pswpout[tog]-pswpout[!tog])*kb_per_page+pero2 )/per,
+	   (  pgpgin [tog]-pgpgin [!tog]             +pero2 )/per,
+	   (  pgpgout[tog]-pgpgout[!tog]             +pero2 )/per,
+	   (inter[tog]-inter[!tog]+pero2)/per,
+	   (ctxt[tog]-ctxt[!tog]+pero2)/per,
+	   (100*duse+divo2)/Div,
+	   (100*dsys+divo2)/Div,
+	   (100*didl+divo2)/Div
+    );
   }
   exit(EXIT_SUCCESS);
 }
