@@ -9,24 +9,22 @@
 #include "proc/devname.h"
 #include "proc/procps.h"
 #include "proc/sysinfo.h"
-#include <ctype.h>
 #include <errno.h>
-#include <fcntl.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <ctype.h>
 #include <string.h>
-#include <sys/ioctl.h>
-#include <sys/mman.h>
-#include <sys/stat.h>
-#include <sys/types.h>
-#include <pwd.h>
-#include <time.h>
 #include <unistd.h>
+#include <time.h>
 #include <utmp.h>
+#include <fcntl.h>
+#include <sys/ioctl.h>
+#include <sys/stat.h>
+#include <sys/mman.h>
 /* #include <sys/param.h>*/	/* for HZ */
 
-static int ignoreuser = 0;	/* for '-u' */
-static proc_t **procs;		/* our snapshot of the process table */
+int ignoreuser = 0;		/* for '-u' */
+proc_t **procs;			/* our snapshot of the process table */
 
 typedef struct utmp utmp_t;
 
@@ -36,7 +34,6 @@ typedef struct utmp utmp_t;
 #   define FROM_STRING "off"
 #endif
 
-/* Uh... same thing as UT_NAMESIZE */
 #define USERSZ (sizeof u->ut_user)
 
 
@@ -90,11 +87,17 @@ static void print_time_ival7(time_t t, int centi_sec, FILE* fout) {
 }
 
 /**** stat the device file to get an idle time */
-static time_t idletime(char *tty) {
-    struct stat sbuf;
-    if (stat(tty, &sbuf) != 0)
+static time_t idletime(utmp_t *u, char *tty) {
+    struct stat terminfo;
+    time_t idle_secs;
+    double uptime_secs;
+    if (stat(tty, &terminfo) != 0)
 	return 0;
-    return time(NULL) - sbuf.st_atime;
+    idle_secs = time(NULL) - terminfo.st_atime;
+    uptime (&uptime_secs, NULL);
+    if ((time_t) uptime_secs < idle_secs)
+	idle_secs = (time_t) uptime_secs;
+    return idle_secs;
 }
 
 /***** 7 character formatted login time */
@@ -119,7 +122,7 @@ static void print_logintime(time_t logt, FILE* fout) {
     if (hour == 0)  hour = 12;
     if (curt - logt > 12*60*60 && logtm->tm_yday != today) {
 	if (curt - logt > 6*24*60*60)
-	    fprintf(fout, " %2d%3s%2d", logtm->tm_mday, month[logtm->tm_mon],
+	    fprintf(fout, " %2d%3s%02d", logtm->tm_mday, month[logtm->tm_mon],
 		    logtm->tm_year % 100);
 	else
 	    fprintf(fout, " %3s%2d%s", weekday[logtm->tm_wday], hour, merid);
@@ -136,43 +139,37 @@ static void print_logintime(time_t logt, FILE* fout) {
  */
 static proc_t *getproc(utmp_t *u, char *tty, int *jcpu, int *found_utpid) {
     int line;
-    proc_t **p, *best = NULL;
-    unsigned uid = ~0U;
+    proc_t **p, *best = NULL, *secondbest = NULL;
 
-    if(!ignoreuser){
-      char buf[UT_NAMESIZE+1];
-      struct passwd *passwd_data;   /* pointer to static data */
-      strncpy(buf,u->ut_user,UT_NAMESIZE);
-      buf[UT_NAMESIZE] = '\0';
-      passwd_data = getpwnam(buf);
-      if(!passwd_data) return NULL;
-      uid = passwd_data->pw_uid;
-      /* OK to have passwd_data go out of scope here */
-    }
     line = tty_to_dev(tty);
     *jcpu = *found_utpid = 0;
-    for(p = procs; *p; p++) {
-	if((**p).pid == u->ut_pid)   *found_utpid = 1;
-	if((**p).tty != line) continue;
-        (*jcpu) += (**p).utime + (**p).stime;
-        if(!ignoreuser && uid != (**p).euid && uid != (**p).ruid) continue;
-        if((**p).pid != (**p).tpgid) continue;
-        if(best && (**p).start_time <= best->start_time) continue;
-    	best = *p;
+    for (p = procs; *p; p++) {
+	if ((**p).pid == u->ut_pid)
+	    *found_utpid = 1;
+	if ((**p).tty == line) {
+	    (*jcpu) += (**p).utime + (**p).stime;
+
+	    secondbest=*p;
+
+	    if ((ignoreuser || !strncmp((**p).euser, u->ut_user, USERSZ)) &&
+		((**p).pid == (**p).tpgid && (!best || (**p).pid > best->pid)))
+		best = *p;
+
+	}
     }
+    if (!best) {return secondbest;}
     return best;
 }
 
 
 /***** showinfo */
 static void showinfo(utmp_t *u, int formtype, int maxcmd, int from) {
-    int jcpu, ut_pid_found;
-    unsigned i;
+    int jcpu, i, ut_pid_found;
     char uname[USERSZ + 1] = "",
 	tty[5 + sizeof u->ut_line + 1] = "/dev/";
     proc_t *best;
 
-    for (i=0; i < sizeof(u->ut_line); i++)	/* clean up tty if garbled */
+    for (i=0; i < sizeof u->ut_line; i++)	/* clean up tty if garbled */
 	if (isalnum(u->ut_line[i]) || (u->ut_line[i]=='/'))
 	    tty[i+5] = u->ut_line[i];
 	else
@@ -194,9 +191,9 @@ static void showinfo(utmp_t *u, int formtype, int maxcmd, int from) {
 	    print_host(u->ut_host, sizeof u->ut_host);
 	print_logintime(u->ut_time, stdout);
 	if (*u->ut_line == ':')			/* idle unknown for xdm logins */
-	    printf(" ?xdm? ");
+	    printf("   ?   ");
 	else
-	    print_time_ival7(idletime(tty), 0, stdout);
+	    print_time_ival7(idletime(u, tty), 0, stdout);
 	print_time_ival7(jcpu/Hertz, (jcpu%Hertz)*(100./Hertz), stdout);
 	if (best) {
 	    int pcpu = best->utime + best->stime;
@@ -208,9 +205,9 @@ static void showinfo(utmp_t *u, int formtype, int maxcmd, int from) {
 	if (from)
 	    print_host(u->ut_host, sizeof u->ut_host);
 	if (*u->ut_line == ':')			/* idle unknown for xdm logins */
-	    printf(" ?xdm? ");
+	    printf("   ?   ");
 	else
-	    print_time_ival7(idletime(tty), 0, stdout);
+	    print_time_ival7(idletime(u, tty), 0, stdout);
     }
     fputs("  ", stdout);
     if (best) {
