@@ -1,20 +1,22 @@
-/* watch -- execute a program repeatedly, displaying output fullscreen
- *
- * Based on the original 1991 'watch' by Tony Rems <rembo@unisoft.com>
- * (with mods and corrections by Francois Pinard).
- *
- * Substantially reworked, new features (differences option, SIGWINCH
- * handling, unlimited command length, long line handling) added Apr 1999 by
- * Mike Coleman <mkc@acm.org>.
- */
+/* watch -- execute a program repeatedly, displaying output fullscreen */
+
+/* $Id */
+
+/* Based on the original 1991 'watch' by Tony Rems <rembo@unisoft.com>
+   (with mods and corrections by Francois Pinard).
+
+   Substantially reworked, new features (differences option, SIGWINCH
+   handling, unlimited command length, long line handling) added Apr 1999 by
+   Mike Coleman <mkc@acm.org>.
+*/
 
 
 #define VERSION "0.2.0"
 
 #include <ctype.h>
 #include <getopt.h>
-#include <ncurses.h>
 #include <signal.h>
+#include <ncurses.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -23,7 +25,7 @@
 #include <unistd.h>
 
 
-static struct option longopts[] =
+struct option longopts[] =
   {
     { "differences", optional_argument, 0, 'd' },
     { "help", no_argument, 0, 'h' },
@@ -32,22 +34,22 @@ static struct option longopts[] =
     { 0, 0, 0, 0 }
   };
 
-static char usage[] = "Usage: %s [-dhnv] [--differences[=cumulative]] [--help] [--interval=<n>] [--version] <command>\n";
+char usage[] = "Usage: %s [-dhnv] [--differences[=cumulative]] [--help] [--interval=<n>] [--version] <command>\n";
 
 
 static char *progname;
 
-static int curses_started = 0;
-static int height=24, width=80;
-static int screen_size_changed=0;
-static int first_screen=1;
+int curses_started = 0;
+int height=24, width=80;
+int screen_size_changed=0;
+int first_screen=1;
 
 
 #define min(x,y) ((x) > (y) ? (y) : (x))
 #define max(x,y) ((x) > (y) ? (x) : (y))
 
 
-static void
+void
 do_usage(void)
 {
   fprintf(stderr, usage, progname);
@@ -55,7 +57,7 @@ do_usage(void)
 }
 
 
-static void
+void
 do_exit(int status) {
   if (curses_started)
     endwin();
@@ -64,24 +66,22 @@ do_exit(int status) {
 
 
 /* signal handler */
-static void
+void
 die(int notused)
 {
-  (void)notused;
   do_exit(0);
 }
 
 
-static void
+void
 winch_handler(int notused)
 {
-  (void)notused;
   screen_size_changed = 1;
 }
 
 
-static void
-get_terminal_size(void)
+void
+get_terminal_size()
 {
   struct winsize w;
   if (ioctl(2, TIOCGWINSZ, &w) == 0)
@@ -186,7 +186,7 @@ main(int argc, char *argv[])
   noecho();
   cbreak();
 
-  for(;;)
+  while (1)
     {
       time_t t = time(NULL);
       char *ts = ctime(&t);
@@ -223,7 +223,7 @@ main(int argc, char *argv[])
 
       for (y=2; y<height; y++)
 	{
-	  int eolseen = 0;
+	  int eolseen = 0, tabpending = 0;
 	  for (x=0; x<width; x++)
 	    {
 	      int c = ' ';
@@ -231,13 +231,20 @@ main(int argc, char *argv[])
 	      
 	      if (!eolseen)
 		{
-		  do
-		    c = getc(p);
-		  while (c != EOF && !isprint(c) && c != '\n');
+		  /* if there is a tab pending, just spit spaces until the
+		     next stop instead of reading characters */
+		  if (!tabpending)
+		    do
+		      c = getc(p);
+		    while (c != EOF && !isprint(c) && c != '\n' && c != '\t');
 		  if (c == '\n')
 		    eolseen = 1;
-		  if (c == EOF || c == '\n')
+		  else if (c == '\t')
+		    tabpending = 1;
+		  if (c == EOF || c == '\n' || c == '\t')
 		    c = ' ';
+		  if (tabpending && (((x + 1) % 8) == 0))
+		    tabpending = 0;
 		}
 	      move(y, x);
 	      if (option_differences)
